import os
import sys
import tempfile
import shutil

from bento.core.platforms.sysconfig \
    import \
        get_scheme
from bento.utils.utils \
    import \
        subst_vars
from bento.installed_package_description \
    import \
        InstalledPkgDescription, iter_files
from bento._config \
    import \
        IPKG_PATH
from bento.commands.core \
    import \
        Command, Option
from bento.commands.mpkg_utils \
    import \
        build_pkg, PackageInfo, MetaPackageInfo, make_mpkg_plist, make_mpkg_description
from bento.utils.utils \
    import \
        MODE_755

def get_default_scheme(pkg_name, py_version_short=None, prefix=None):
    if py_version_short is None:
        py_version_short = ".".join([str(i) for i in sys.version_info[:2]])
    if prefix is None:
        prefix = sys.exec_prefix
    scheme, _ = get_scheme(sys.platform)
    scheme["prefix"] = scheme["eprefix"] = prefix
    scheme["pkgname"] = pkg_name
    scheme["py_version_short"] = py_version_short
    ret = {}
    for k in scheme:
        ret[k] = subst_vars(scheme[k], scheme)
    return ret

class BuildMpkgCommand(Command):
    long_descr = """\
Purpose: build Mac OS X mpkg
Usage:   bentomaker build_mpkg [OPTIONS]"""
    short_descr = "build mpkg."
    common_options = Command.common_options \
                        + [Option("--output-dir",
                                  help="Output directory", default="dist"),
                           Option("--output-file",
                                  help="Output filename")]

    def run(self, ctx):
        argv = ctx.command_argv
        p = ctx.options_context.parser
        o, a = p.parse_args(argv)
        if o.help:
            p.print_help()
            return

        root = ctx.top_node
        while root.height() > 0:
            root = root.parent

        default_scheme = get_default_scheme(ctx.pkg.name)
        default_prefix = default_scheme["prefix"]
        default_sitedir = default_scheme["sitedir"]

        n = ctx.build_node.make_node(IPKG_PATH)
        ipkg = InstalledPkgDescription.from_file(n.abspath())
        name = ipkg.meta["name"]
        version = ipkg.meta["version"]
        py_short = ".".join([str(i) for i in sys.version_info[:2]])
        if o.output_file is None:
            mpkg_name = "%s-%s-py%s.mpkg" % (name, version, py_short)
        else:
            mpkg_name = o.output_file

        categories = set()
        file_sections = ipkg.resolve_paths(ctx.build_node)
        for kind, source, target in iter_files(file_sections):
            categories.add(kind)

        # Mpkg metadata
        mpkg_root = os.path.join(os.getcwd(), o.output_dir, mpkg_name)
        mpkg_cdir = os.path.join(mpkg_root, "Contents")
        if os.path.exists(mpkg_root):
            shutil.rmtree(mpkg_root)
        os.makedirs(mpkg_cdir)
        f = open(os.path.join(mpkg_cdir, "PkgInfo"), "w")
        try:
            f.write("pmkrpkg1")
        finally:
            f.close()
        mpkg_info = MetaPackageInfo.from_ipkg(ipkg)

        purelib_pkg = "%s-purelib-%s-py%s.pkg" % (name, version, py_short)
        scripts_pkg = "%s-scripts-%s-py%s.pkg" % (name, version, py_short)
        datafiles_pkg = "%s-datafiles-%s-py%s.pkg" % (name, version, py_short)
        mpkg_info.packages = [purelib_pkg, scripts_pkg, datafiles_pkg]
        make_mpkg_plist(mpkg_info, os.path.join(mpkg_cdir, "Info.plist"))

        mpkg_rdir = os.path.join(mpkg_root, "Contents", "Resources")
        os.makedirs(mpkg_rdir)
        make_mpkg_description(mpkg_info, os.path.join(mpkg_rdir, "Description.plist"))

        # Package the stuff which ends up into site-packages
        pkg_root = os.path.join(mpkg_root, "Contents", "Packages", purelib_pkg)
        build_pkg_from_temp(ctx, ipkg, pkg_root, root, "/", ["pythonfiles"], "Pure Python modules and packages")

        pkg_root = os.path.join(mpkg_root, "Contents", "Packages", scripts_pkg)
        build_pkg_from_temp(ctx, ipkg, pkg_root, root, "/", ["executables"], "Scripts and binaries")

        pkg_root = os.path.join(mpkg_root, "Contents", "Packages", datafiles_pkg)
        build_pkg_from_temp(ctx, ipkg, pkg_root, root, "/", ["bentofiles", "datafiles"], "Data files")

def build_pkg_from_temp(ctx, ipkg, pkg_root, root_node, install_root, categories, description=None):
    d = tempfile.mkdtemp()
    try:
        tmp_root = root_node.make_node(d)
        prefix_node = tmp_root.make_node(root_node.make_node(sys.exec_prefix).path_from(root_node))
        prefix = eprefix = prefix_node.abspath()
        ipkg.update_paths({"prefix": prefix, "eprefix": eprefix})
        file_sections = ipkg.resolve_paths(ctx.build_node)
        for kind, source, target in iter_files(file_sections):
            if kind in categories:
                #if not os.path.exists(target.parent.abspath()):
                #    os.makedirs(os.path.dirname(target))
                target.parent.mkdir()
                shutil.copy(source.abspath(), target.abspath())
                if kind == "executables":
                    os.chmod(target.abspath(), MODE_755)

        pkg_name = os.path.splitext(os.path.basename(pkg_root))[0]
        pkg_info = PackageInfo(pkg_name=pkg_name,
            prefix=install_root, source_root=d, pkg_root=pkg_root, description=description)
        build_pkg(pkg_info)
    finally:
        shutil.rmtree(d)



import os
import sys
import warnings
import posixpath

import os.path as op

from bento.compat.api \
    import \
        relpath
from bento.conv \
    import \
        find_package, distutils_to_package_description
from bento.utils.utils \
    import \
        pprint, extract_exception
from bento.core.pkg_objects \
    import \
        DataFiles

from bento.errors \
    import \
        UsageException, ConvertionError
from bento.convert.utils \
    import \
        canonalize_path

import bento.errors

# ====================================================
# Code to convert existing setup.py to bento.info
# ====================================================
DIST_GLOBAL = None
PACKAGE_OBJECTS = None

def canonalized_path_to_package(path):
    return path.replace(posixpath.sep, ".")

def _convert_numpy_data_files(top_node, source_dir, files):
    """Convert data_files pairs to the common format we use.

    numpy.distutils internally keeps data as a pair (package_path, files_list),
    where files_list is relative to the top source path. We convert this

    Parameters
    ----------
    top_node: node
        top directory of the source tree (as a node).
    source_dir: str
        the source directory (as a path string, relative to top node).
    files: seq
        list of files (relative to top source)

    Returns
    -------
    pkg_name: str
        name of the package
    source_dir: str
        source directory
    target_dir: str
        target directory
    files: seq
        list of files (relative to source directory)
    """
    source_node = top_node.find_node(source_dir)
    if source_node is None:
        raise ConvertionError("directory %r not found" % source_dir)
    nodes = []
    for f in files:
        node = top_node.find_node(f)
        if node is None:
            raise ConvertionError("file %s refered in data_files not found" % f)
        nodes.append(node)
    pkg_name = canonalized_path_to_package(source_dir)
    target_dir = canonalize_path(op.join("$sitedir", source_dir))
    return pkg_name, source_dir, target_dir, [node.path_from(source_node) for node in nodes]

class _PackageObjects(object):
    """This private class is used to record the distribution data in our
    instrumented setup."""
    def __init__(self, monkey_patch_mode="distutils"):
        self.package_data = {}
        self.extra_source_files = {}
        self.dist_data_files = []
        self.data_files = []

        self.monkey_patch_mode = monkey_patch_mode
        self.build_lib = None

    def iter_data_files(self, top_node):
        for pkg_name, source_dir, build_dir, files in self.data_files:
            if files:
                target_dir = relpath(build_dir, self.build_lib)
                yield pkg_name, source_dir, op.join("$sitedir", target_dir), files
        if self.monkey_patch_mode == "setuptools_numpy":
            if self.dist_data_files:
                assert len(self.dist_data_files[0]) == 2, "Unhandled data files representation"
                for source_dir, files in self.dist_data_files:
                    yield _convert_numpy_data_files(top_node, source_dir, files)
        else:
            yield "", ".", "$sitedir", self.dist_data_files

# XXX: this is where the magic happens. This is highly dependent on the
# setup.py, whether it uses distutils, numpy.distutils, setuptools and whatnot.
def monkey_patch(top_node, type, filename):
    supported = ["distutils", "numpy_distutils", "setuptools", "setuptools_numpy"]

    if type == "distutils":
        from distutils.core import setup as old_setup
        from distutils.command.build_py import build_py as old_build_py
        from distutils.command.sdist import sdist as old_sdist
        from distutils.dist import Distribution as _Distribution
        from distutils.filelist import FileList
    elif type == "setuptools":
        from setuptools import setup as old_setup
        from setuptools.command.build_py import build_py as old_build_py
        from setuptools.command.sdist import sdist as old_sdist
        from distutils.dist import Distribution as _Distribution
        from distutils.filelist import FileList
    elif type == "numpy_distutils":
        import numpy.distutils
        import distutils.core
        from numpy.distutils.core import setup as old_setup
        from numpy.distutils.command.build_py import build_py as old_build_py
        from numpy.distutils.command.sdist import sdist as old_sdist
        from numpy.distutils.numpy_distribution import NumpyDistribution as _Distribution
        from distutils.filelist import FileList
    elif type == "setuptools_numpy":
        import setuptools
        import numpy.distutils
        import distutils.core
        from numpy.distutils.core import setup as old_setup
        from numpy.distutils.command.build_py import build_py as old_build_py
        from numpy.distutils.command.sdist import sdist as old_sdist
        from numpy.distutils.numpy_distribution import NumpyDistribution as _Distribution
        from distutils.filelist import FileList
    else:
        raise UsageException("Unknown converter: %s (known converters are %s)" % 
                         (type, ", ".join(supported)))

    def get_extra_source_files():
        """Return the list of files included in the tarball."""
        # FIXME: handle redundancies between data files, package data files
        # (i.e. installed data files) and files included as part of modules,
        # packages, extensions, .... Given the giant mess that distutils makes
        # of things here, it may not be possible to get everything right,
        # though.
        dist = _Distribution()
        sdist = old_sdist(dist)
        sdist.initialize_options()
        sdist.finalize_options()
        sdist.manifest_only = True
        sdist.filelist = FileList()
        sdist.distribution.script_name = filename
        sdist.get_file_list()
        return sdist.filelist.files


    def new_setup(**kw):
        global DIST_GLOBAL, PACKAGE_OBJECTS
        package_objects = _PackageObjects(monkey_patch_mode=type)

        package_dir = kw.get("package_dir", None)
        if package_dir:
            keys = list(package_dir.keys())
            if len(keys) > 1:
                raise ConvertionError("setup call with package_dir=%r argument is not supported !" \
                                      % package_dir)
            elif len(keys) == 1:
                if package_dir.values()[0] != '':
                    raise ConvertionError("setup call with package_dir=%r argument is not supported !" \
                                          % package_dir)

        cmdclass = kw.get("cmdclass", {})
        try:
            _build_py = cmdclass["build_py"]
        except KeyError:
            _build_py = old_build_py

        class build_py_recorder(_build_py):
            def run(self):
                _build_py.run(self)

                package_objects.build_lib = self.build_lib
                package_objects.extra_source_files = get_extra_source_files()

                # This is simply the data_files argument passed to setup
                if self.distribution.data_files is not None:
                    package_objects.dist_data_files.extend(self.distribution.data_files)
                # those are created from package_data stuff (the stuff included
                # if include_package_data=True as well)
                package_objects.data_files = self.data_files

        cmdclass["build_py"] = build_py_recorder
        kw["cmdclass"] = cmdclass

        dist = old_setup(**kw)
        DIST_GLOBAL = dist
        PACKAGE_OBJECTS = package_objects
        return dist

    if type == "distutils":
        import distutils.core
        distutils.core.setup = new_setup
    elif type == "setuptools":
        import distutils.core
        import setuptools
        distutils.core.setup = new_setup
        setuptools.setup = new_setup
    elif type == "setuptools_numpy":
        numpy.distutils.core.setup = new_setup
        setuptools.setup = new_setup
        distutils.core.setup = new_setup
    elif type == "numpy_distutils":
        numpy.distutils.core.setup = new_setup
        distutils.core.setup = new_setup
    else:
        raise UsageException("Unknown converter: %s (known converters are %s)" % 
                         (type, ", ".join(supported)))

def analyse_setup_py(filename, setup_args, verbose=False):
    # This is the dirty part: we run setup.py inside this process, and pass
    # data back through global variables. Not sure if there is a better way to
    # do this
    if verbose:
        pprint('PINK', "======================================================")
        pprint('PINK', " Analysing %s (running %s) .... " % (filename, filename))

    # exec_globals contains the globals used to execute the setup.py
    exec_globals = {}
    exec_globals.update(globals())
    # Some setup.py files call setup from their main, so execute them as if
    # they were the main script
    exec_globals["__name__"] = "__main__"
    exec_globals["__file__"] = op.abspath(filename)

    _saved_argv = sys.argv[:]
    _saved_sys_path = sys.path
    try:
        try:
            sys.argv = [filename] + setup_args + ["build_py"]
            # XXX: many packages import themselves to get version at build
            # time, and setuptools screw this up by inserting stuff first. Is
            # there a better way ?
            sys.path.insert(0, op.dirname(filename))
            fid = open(filename, "r")
            try:
                exec(fid.read(), exec_globals)
                if type == "distutils" and "setuptools" in sys.modules and verbose:
                    pprint("YELLOW", "Setuptools detected in distutils mode !!!")
            finally:
                fid.close()
        except ConvertionError:
            raise
        except Exception:
            e = extract_exception()
            pprint('RED', "Got exception: %s" % e)
            raise
    finally:
        sys.argv = _saved_argv
        sys.path = _saved_sys_path

    live_objects = PACKAGE_OBJECTS
    dist = DIST_GLOBAL
    if dist is None:
        raise ValueError("setup monkey-patching failed")
    else:
        if verbose:
            pprint('PINK', " %s analyse done " % filename)
            pprint('PINK', "======================================================")
        return dist, live_objects

def build_pkg(dist, package_objects, top_node):
    pkg = distutils_to_package_description(dist)
    modules = []
    for m in pkg.py_modules:
        if isinstance(m, basestring):
            modules.append(m)
        else:
            warnings.warn("The module %s it not understood" % str(m))
    pkg.py_modules = modules

    path_options = []
    data_sections = {}

    extra_source_files = []
    if package_objects.extra_source_files:
        extra_source_files.extend([canonalize_path(f) 
                                  for f in package_objects.extra_source_files])

    for pkg_name, source_dir, target_dir, files in package_objects.iter_data_files(top_node):
        if len(files) > 0:
            if len(pkg_name) > 0:
                name = "%s_data" % pkg_name.replace(".", "_")
            else:
                name = "dist_data"
            source_dir = canonalize_path(source_dir)
            target_dir = canonalize_path(target_dir)
            files = [canonalize_path(f) for f in files]
            data_sections[name] = DataFiles(name, files, target_dir, source_dir)
    pkg.data_files.update(data_sections)

    if dist.scripts:
        name = "%s_scripts" % pkg.name
        target_dir = "$bindir"
        pkg.data_files[name] = DataFiles(name, dist.scripts, target_dir, ".")

    # numpy.distutils bug: packages are appended twice to the Distribution
    # instance, so we prune the list here
    pkg.packages = sorted(list(set(pkg.packages)))
    options = {"path_options": path_options}

    pkg.extra_source_files = sorted(prune_extra_files(extra_source_files, pkg, top_node))

    return pkg, options

def prune_extra_files(files, pkg, top_node):
    package_files = []
    for p in pkg.packages:
        package_files.extend(find_package(p, top_node))
    package_files = [canonalize_path(f) for f in package_files]

    data_files = []
    for data_section in pkg.data_files.values():
        data_files.extend([posixpath.join(data_section.source_dir, f) for f in data_section.files])

    redundant = package_files + data_files + pkg.py_modules

    return prune_file_list(files, redundant)

def detect_monkeys(setup_py, show_output, log):
    from bento.convert.utils import \
        test_distutils, test_setuptools, test_numpy, test_setuptools_numpy, \
        test_can_run

    if not test_can_run(setup_py, show_output, log):
        raise bento.errors.SetupCannotRun()

    def print_delim(string):
        if show_output:
            pprint("YELLOW", string)

    print_delim("----------------- Testing distutils ------------------")
    use_distutils = test_distutils(setup_py, show_output, log)
    print_delim("----------------- Testing setuptools -----------------")
    use_setuptools = test_setuptools(setup_py, show_output, log)
    print_delim("------------ Testing numpy.distutils -----------------")
    use_numpy = test_numpy(setup_py, show_output, log)
    print_delim("--- Testing numpy.distutils patched by setuptools ----")
    use_setuptools_numpy = test_setuptools_numpy(setup_py, show_output, log)
    print_delim("Is distutils ? %s" % use_distutils)
    print_delim("Is setuptools ? %s" % use_setuptools)
    print_delim("Is numpy distutils ? %s" % use_numpy)
    print_delim("Is setuptools numpy ? %s" % use_setuptools_numpy)

    if use_distutils and not (use_setuptools or use_numpy or use_setuptools_numpy):
        return "distutils"
    elif use_setuptools  and not (use_numpy or use_setuptools_numpy):
        return "setuptools"
    elif use_numpy  and not use_setuptools_numpy:
        return "numpy"
    elif use_setuptools_numpy:
        return "setuptools_numpy"
    else:
        raise ValueError("Unsupported converter")

def prune_file_list(files, redundant):
    """Prune a list of files relatively to a second list.

    Return a subsequence of `files' which contains only files not in
    `redundant'

    Parameters
    ----------
    files: seq
        list of files to prune.
    redundant: seq
        list of candidate files to prune.
    """
    files_set = set([posixpath.normpath(f) for f in files])
    redundant_set = set([posixpath.normpath(f) for f in redundant])

    return list(files_set.difference(redundant_set))



import bento

from bento.commands.core \
    import \
        USAGE, fill_string

def get_usage(global_context):
    ret = [USAGE % {"name": "bentomaker",
                    "version": bento.__version__}]
    ret.append("Bento commands:")

    commands = []
    cmd_names = sorted(global_context.command_names())
    for name in cmd_names:
        v = global_context.retrieve_command(name)
        doc = v.short_descr
        if doc is None:
            doc = "undocumented"
        header = "  %s" % name
        commands.append((header, doc))

    minlen = max([len(header) for header, hlp in commands]) + 2
    for header, hlp in commands:
        ret.append(fill_string(header, minlen) + hlp)
    return "\n".join(ret)




from os.path \
    import \
        abspath, splitunc, curdir, sep, pardir, join

def relpath(path, start=curdir):
    """Return a relative version of a path"""

    if not path:
        raise ValueError("no path specified")
    start_list = abspath(start).split(sep)
    path_list = abspath(path).split(sep)
    if start_list[0].lower() != path_list[0].lower():
        unc_path, rest = splitunc(path)
        unc_start, rest = splitunc(start)
        if bool(unc_path) ^ bool(unc_start):
            raise ValueError("Cannot mix UNC and non-UNC paths (%s and %s)"
                                                                % (path, start))
        else:
            raise ValueError("path is on drive %s, start on drive %s"
                                                % (path_list[0], start_list[0]))
    # Work out how much of the filepath is shared by start and path.
    for i in range(min(len(start_list), len(path_list))):
        if start_list[i].lower() != path_list[i].lower():
            break
    else:
        i += 1

    rel_list = [pardir] * (len(start_list)-i) + path_list[i:]
    if not rel_list:
        return curdir
    return join(*rel_list)



import re

from waflib.Tools.c_config import SNIP_EMPTY_PROGRAM
from waflib.Configure import conf

ARCHS = ["i386", "x86_64", "ppc", "ppc64"]

FILE_MACHO_RE = re.compile("Mach-O.*object ([a-zA-Z_0-9]+)")

@conf
def check_cc_arch(conf):
    env = conf.env
    archs = []

    for arch in ARCHS:
        env.stash()
        try:
            env.append_value('CFLAGS', ['-arch', arch])
            env.append_value('LINKFLAGS', ['-arch', arch])
            try:
                conf.check_cc(fragment=SNIP_EMPTY_PROGRAM, msg="Checking for %r suport" % arch)
                archs.append(arch)
            except conf.errors.ConfigurationError:
                pass
        finally:
            env.revert()

    env["ARCH_CC"] = archs

#def detect_arch(filename):

@conf
def check_cc_default_arch(conf):
    start_msg = "Checking for default CC arch"
    fragment = SNIP_EMPTY_PROGRAM
    output_var = "DEFAULT_CC_ARCH"

    return _check_default_arch(conf, start_msg, fragment, output_var)

@conf
def check_cxx_default_arch(conf):
    start_msg = "Checking for default CXX arch"
    fragment = SNIP_EMPTY_PROGRAM
    output_var = "DEFAULT_CXX_ARCH"

    return _check_default_arch(conf, start_msg, fragment, output_var)

@conf
def check_fc_default_arch(conf):
    start_msg = "Checking for default FC arch"
    fragment = """\
      program main
      end
"""
    output_var = "DEFAULT_FC_ARCH"
    compile_filename = 'test.f'
    features = "fc fcprogram"

    return _check_default_arch(conf, start_msg, fragment, output_var, compile_filename, features)

@conf
def _check_default_arch(conf, start_msg, fragment, output_var, compile_filename="test.c", features="c cprogram"):
    env = conf.env

    if not "FILE_BIN" in conf.env:
        file_bin = conf.find_program(["file"], var="FILE_BIN")
    else:
        file_bin = conf.env.FILE_BIN

    conf.start_msg(start_msg)
    ret = conf.check_cc(fragment=fragment, compile_filename=compile_filename, features=features)
    task_gen = conf.test_bld.groups[0][0]
    obj_filename = task_gen.tasks[0].outputs[0].abspath()
    out = conf.cmd_and_log([file_bin, obj_filename])
    m = FILE_MACHO_RE.search(out)
    if m is None:
        conf.fatal("Could not determine arch from output %r" % out)
    else:
        default_arch = m.group(1)
        conf.env[output_var] = default_arch
        conf.end_msg(default_arch)



import os
import sys
import string

import os.path as op

from bento.errors \
    import \
        InvalidPackage
from bento.utils.utils \
    import \
        is_string, subst_vars
from bento.compat.api \
    import \
        defaultdict
from bento.core.node_package \
    import \
        NodeRepresentation
from bento.commands.build \
    import \
        _config_content
from bento.core.meta \
    import \
        PackageMetadata
from bento.commands.script_utils \
    import \
        create_posix_script, create_win32_script
from bento._config \
    import \
        CONFIGURED_STATE_DUMP
from bento.commands.configure \
    import \
        _compute_scheme
from bento.commands.build \
    import \
        SectionWriter
from bento.commands.install \
    import \
        copy_installer
from bento.installed_package_description \
    import \
        InstalledSection

class DummyContextManager(object):
    def __init__(self, pre, post):
        self.pre = pre
        self.post = post

    def __enter__(self):
        self.pre()

    def __exit__(self, *a, **kw):
        self.post()

class CmdContext(object):
    def __init__(self, global_context, command_argv, options_context, pkg, run_node):
        self._global_context = global_context
        self.pkg = pkg

        self.options_context = options_context
        o, a = options_context.parser.parse_args(command_argv)
        if o.help:
            self.help = True
        else:
            self.help = False

        self.command_argv = command_argv

        # All of run/top/cur nodes are set to the same value for the base
        # context: without a bento.info available, neither build directory, nor
        # out-of-tree concepts make much sense.

        # CWD node
        self.run_node = run_node
        # Top source node (the one containing the top bento.info)
        self.top_node = run_node
        # cur_node refers to the current path when recursing into sub directories
        self.cur_node = run_node

        # Recursive related members
        self.local_node = None
        self.local_pkg = None

    def recurse_manager(self, local_node):
        """
        Return a dummy object to use for recurse if one wants to use context
        manager. Example::

            with context.recurse_manager(local_node):
                func(context)
        """
        return DummyContextManager(lambda: self.pre_recurse(local_node),
                                   lambda: self.post_recurse())

    def pre_recurse(self, local_node):
        """
        Note
        ----
        Every call to pre_recurse should be followed by a call to post_recurse.

        Calling pre_recurse for the top hook node must work as well (but could
        do nothing)
        """
        if local_node == self.run_node:
            self.local_node = self.run_node
            return
        else:
            if not local_node.is_src():
                raise IOError("node %r is not in source tree !" % local_node.abspath())
            self.local_node = local_node

            def _get_sub_package():
                relative_pos = local_node.path_from(self.top_or_sub_directory_node)
                local_bento = self.top_node.find_node(op.join(relative_pos, "bento.info"))
                k = local_bento.path_from(self.run_node)
                if k is None:
                    raise IOError("%r not found" % os.path.join(local_node.abspath(), "bento.info"))
                else:
                    return self.pkg.subpackages.get(k, None)
            self.local_pkg = _get_sub_package()

    def post_recurse(self):
        # Setting those to None is not strictly necessary, but this makes
        # things more consistent for debugging (context state exactly same
        # before pre_recurse and after post_recurse
        self.local_node = None
        self.local_pkg = None

    def get_parsed_arguments(self):
        return self.options_context.parser.parse_args(self.command_argv)

    def retrieve_scheme(self):
        return self._global_context.retrieve_scheme()

    def retrieve_configured_scheme(self):
        configure_argv = self._global_context.retrieve_command_argv("configure")
        return self._global_context.retrieve_configured_scheme(configure_argv)

    # This is run before the associated command pre-hooks
    def init(self):
        pass

    # This is run after the associated command pre-hooks, but before the command run function
    def configure(self):
        pass

    # This is run after the associated command post_hooks
    def finish(self):
        pass

class ContextWithBuildDirectory(CmdContext):
    def __init__(self, *a, **kw):
        super(ContextWithBuildDirectory, self).__init__(*a, **kw)
        self.build_root = self.run_node.make_node("build")

        # TODO: kept for compatibility. Remove it ?
        assert self.run_node is not None
        self.top_node = self.run_node._ctx.srcnode
        self.build_node = self.run_node._ctx.bldnode

        if self.pkg.sub_directory:
            self.top_or_sub_directory_node = self.top_node.make_node(self.pkg.sub_directory)
            self.build_or_sub_directory_node = self.build_node.make_node(self.pkg.sub_directory)
        else:
            self.top_or_sub_directory_node = self.top_node
            self.build_or_sub_directory_node = self.build_node

    def make_source_node(self, path):
        n = self.top_node.find_node(path)
        if n is None:
            raise IOError("file %s not found" % (op.join(self.top_node.abspath(), path)))
        else:
            return n

    def make_build_node(self, path):
        n = self.build_node.make_node(path)
        n.parent.mkdir()
        return n

class ConfigureContext(ContextWithBuildDirectory):
    pass

class _Dummy(object):
    pass

class _RegistryBase(object):
    """A simple registry of sets of callbacks, one set per category."""
    def __init__(self):
        self._callbacks = {}
        self.categories = _Dummy()

    def register_category(self, category, default_builder):
        if category in self._callbacks:
            raise ValueError("Category %r already registered" % category)
        else:
            self._callbacks[category] = defaultdict(lambda: default_builder)
            setattr(self.categories, category, _Dummy())

    def register_callback(self, category, name, builder):
        c = self._callbacks.get(category, None)
        if c is not None:
            c[name] = builder
            cat = getattr(self.categories, category)
            setattr(cat, name, builder)
        else:
            raise ValueError("category %s is not registered yet" % category)

    def callback(self, category, name):
        if not category in self._callbacks:
            raise ValueError("Unregistered category %r" % category)
        else:
            return self._callbacks[category][name]

    def default_callback(self, category, *a, **kw):
        if not category in self._callbacks:
            raise ValueError("Unregistered category %r" % category)
        else:
            return self._callbacks[category].default_factory()(*a, **kw)

class BuilderRegistry(_RegistryBase):
    builder = _RegistryBase.callback

class ISectionRegistry(_RegistryBase):
    registrer = _RegistryBase.callback

class OutputRegistry(object):
    def __init__(self, categories=None):
        self.categories = {}
        self.installed_categories = {}
        if categories:
            for category, installed_category in categories:
                self.register_category(category, installed_category)

    def register_category(self, category, installed_category):
        if category in self.categories:
            raise ValueError("Category %r already registered")
        else:
            self.categories[category] = {}
            self.installed_categories[category] = installed_category

    def register_outputs(self, category, name, nodes, from_node, target_dir):
        if not category in self.categories:
            raise ValueError("Unknown category %r" % category)
        else:
            cat = self.categories[category]
            if name in cat:
                raise ValueError("Outputs for categoryr=%r and name=%r already registered" % (category, name))
            else:
                cat[name] = (nodes, from_node, target_dir)

    def iter_category(self, category):
        if not category in self.categories:
            raise ValueError("Unknown category %r" % category)
        else:
            for k, v in self.categories[category].items():
                yield k, v[0], v[1], v[2]

    def iter_over_category(self):
        for category in self.categories:
            for name, nodes, from_node, target_dir in self.iter_category(category):
                yield category, name, nodes, from_node, target_dir

def _generic_iregistrer(category, name, nodes, from_node, target_dir):
    source_dir = os.path.join("$_srcrootdir", from_node.bldpath())
    files = [n.path_from(from_node) for n in nodes]
    return InstalledSection.from_source_target_directories(
        category, name, source_dir, target_dir, files)

def fill_metadata_template(content, metadata):
    tpl = string.Template(content)

    def _safe_repr(val):
        # FIXME: actually not safe at all. Needs to escape and all.
        if is_string(val):
            if len(val.splitlines()) > 1:
                return '"""%s"""' % (val,)
            else:
                return '"%s"' % (val,)
        else:
            return repr(val)

    meta_dict = dict((k.upper(), _safe_repr(v)) for k, v in metadata.items())

    return tpl.substitute(meta_dict)

def _write_template(template_node, metadata):
    source_content = template_node.read()
    output_content = fill_metadata_template(source_content, metadata)

    output = template_node.change_ext("")
    output.safe_write(output_content)
    return output

def write_template(top_node, template_file, package, additional_metadata=None):
    if additional_metadata is None:
        additional_metadata = {}

    source = top_node.find_node(template_file)
    if source is None:
        raise InvalidPackage("File %r not found (defined in 'MetaTemplateFile' field)" \
                             % (package.meta_template_file,))
    package_metadata = PackageMetadata.from_package(package)
    meta = dict((k, getattr(package_metadata, k)) for k in package_metadata.metadata_attributes)
    meta.update(additional_metadata)
    return _write_template(source, meta)

class BuildContext(ContextWithBuildDirectory):
    def __init__(self, global_context, command_argv, options_context, pkg, run_node):
        super(BuildContext, self).__init__(global_context, command_argv, options_context, pkg, run_node)
        self.builder_registry = BuilderRegistry()
        self.section_writer = SectionWriter()

        o, a = self.options_context.parser.parse_args(command_argv)
        if o.inplace:
            self.inplace = True
        else:
            self.inplace = False
        # Builders signature:
        #   - first argument: name, str. Name of the entity to be built
        #   - second argument: object. Value returned by
        #   NodePackage.iter_category for this category

        # TODO: # Refactor builders so that they directoy register outputs
        # instead of returning stuff to be registered (to allow for "delayed"
        # registration)
        def data_section_builder(name, section):
            return name, section.nodes, section.ref_node, section.target_dir

        def package_builder(name, node_py_package):
            return name, node_py_package.nodes, self.top_or_sub_directory_node, "$sitedir"

        def module_builder(name, node):
            return name, [node], self.top_or_sub_directory_node, "$sitedir"

        def script_builder(name, executable):
            scripts_node = self.build_node.make_node("scripts-%s" % sys.version[:3])
            scripts_node.mkdir()
            if sys.platform == "win32":
                nodes = create_win32_script(name, executable, scripts_node)
            else:
                nodes = create_posix_script(name, executable, scripts_node)
            return name, nodes, scripts_node, "$bindir"

        self.builder_registry.register_category("datafiles", data_section_builder)
        self.builder_registry.register_category("packages", package_builder)
        self.builder_registry.register_category("modules", module_builder)
        self.builder_registry.register_category("scripts", script_builder)

        if self.pkg.sub_directory is not None:
            sub_directory_node = self.top_node.find_node(self.pkg.sub_directory)
        else:
            sub_directory_node = None
        self._node_pkg = NodeRepresentation(run_node, self.top_node, sub_directory_node)
        self._node_pkg.update_package(pkg)

        categories = (("packages", "pythonfiles"), ("modules", "pythonfiles"), ("datafiles", "datafiles"),
                      ("scripts", "executables"), ("extensions", "extensions"),
                      ("compiled_libraries", "compiled_libraries"))
        self.outputs_registry = OutputRegistry(categories)

        self.isection_registry = ISectionRegistry()
        self.isection_registry.register_category("extensions", _generic_iregistrer)
        self.isection_registry.register_category("compiled_libraries", _generic_iregistrer)
        self.isection_registry.register_category("packages", _generic_iregistrer)
        self.isection_registry.register_category("modules", _generic_iregistrer)
        self.isection_registry.register_category("datafiles", _generic_iregistrer)
        self.isection_registry.register_category("scripts", _generic_iregistrer)

        self._current_default_section = 0

        self._meta = {}

    def register_metadata(self, name, value):
        self._meta[name] = value

    def register_category(self, category_name, category_type="pythonfiles"):
        self.outputs_registry.register_category(category_name, category_type)
        self.isection_registry.register_category(category_name, _generic_iregistrer)

    def register_outputs_simple(self, nodes, from_node=None, target_dir='$sitedir'):
        category_name = "hook_registered"
        section_name = "hook_registered%d" % self._current_default_section

        if not category_name in self.outputs_registry.categories:
            self.register_category(category_name)
        self.register_outputs(category_name, section_name, nodes, from_node, target_dir)

        self._current_default_section += 1

    def register_outputs(self, category_name, section_name, nodes, from_node=None, target_dir="$sitedir"):
        if from_node is None:
            from_node = self.build_node
        self.outputs_registry.register_outputs(category_name, section_name, nodes, from_node, target_dir)

    def _compute_extension_name(self, extension_name):
        if self.local_node is None:
            raise ValueError("Forgot to call pre_recurse ?")
        if self.local_node != self.top_node:
            parent = self.local_node.srcpath().split(os.path.sep)
            return ".".join(parent + [extension_name])
        else:
            return extension_name

    def register_builder(self, extension_name, builder):
        full_name = self._compute_extension_name(extension_name)
        self.builder_registry.register_callback("extensions", full_name, builder)

    def default_builder(self, extension, **kw):
        return self.builder_registry.default_callback(
                                        "extensions",
                                        extension,
                                        **kw)

    def tweak_extension(self, extension_name, **kw):
        def _builder(extension):
            return self.default_builder(extension, **kw)
        full_name = self._compute_extension_name(extension_name)
        return self.builder_registry.register_callback("extensions", full_name, _builder)

    def tweak_library(self, lib_name, **kw):
        def _builder(lib_name):
            return self.default_library_builder(lib_name, **kw)
        relpos = self.local_node.path_from(self.top_node)
        full_name = os.path.join(relpos, lib_name).replace(os.sep, ".")
        return self.builder_registry.register_callback("compiled_libraries", full_name, _builder)

    def default_library_builder(self, library, **kw):
        return self.builder_registry.default_callback(
                                        "compiled_libraries",
                                        library,
                                        **kw)

    def disable_extension(self, extension_name):
        def nobuild(extension):
            pass
        self.register_builder(extension_name, nobuild)

    def register_compiled_library_builder(self, clib_name, builder):
        relpos = self.local_node.path_from(self.top_node)
        full_name = os.path.join(relpos, clib_name).replace(os.sep, ".")
        self.builder_registry.register_callback("compiled_libraries", full_name, builder)

    def compile(self):
        for category in ("packages", "modules", "datafiles"):
            for name, value in self._node_pkg.iter_category(category):
                builder = self.builder_registry.builder(category, name)
                name, nodes, from_node, target_dir = builder(name, value)
                self.outputs_registry.register_outputs(category, name, nodes, from_node, target_dir)

        category = "scripts"
        for name, executable in self.pkg.executables.items():
            builder = self.builder_registry.builder(category, name)
            name, nodes, from_node, target_dir = builder(name, executable)
            self.outputs_registry.register_outputs(category, name, nodes, from_node, target_dir)

        if self.pkg.config_py:
            content = _config_content(self.retrieve_configured_scheme())
            target_node = self.build_node.make_node(self.pkg.config_py)
            target_node.parent.mkdir()
            target_node.safe_write(content)
            self.outputs_registry.register_outputs("modules", "bento_config", [target_node],
                                                   self.build_node, "$sitedir")

        if self.pkg.meta_template_files:
            target_nodes = []
            for template in self.pkg.meta_template_files:
                target_node = write_template(self.top_node, template, self.pkg, self._meta)
                target_nodes.append(target_node)
            self.outputs_registry.register_outputs("modules", "meta_from_template", target_nodes,
                                               self.build_node, "$sitedir")
    def post_compile(self):
        # Do the output_registry -> installed sections registry convertion
        section_writer = self.section_writer

        for category, name, nodes, from_node, target_dir in self.outputs_registry.iter_over_category():
            installed_category = self.outputs_registry.installed_categories[category]
            if installed_category in section_writer.sections:
                sections = section_writer.sections[installed_category]
            else:
                sections = section_writer.sections[installed_category] = {}
            registrer = self.isection_registry.registrer(category, name)
            sections[name] = registrer(installed_category, name, nodes, from_node, target_dir)

        # FIXME: this is quite stupid.
        if self.inplace:
            scheme = self.retrieve_scheme()
            scheme["prefix"] = scheme["eprefix"] = self.run_node.abspath()
            scheme["sitedir"] = self.run_node.abspath()

            if self.pkg.config_py:
                target_node = self.build_node.find_node(self.pkg.config_py)
            else:
                target_node = None

            def _install_node(category, node, from_node, target_dir):
                installed_path = subst_vars(target_dir, scheme)
                target = os.path.join(installed_path, node.path_from(from_node))
                copy_installer(node.path_from(self.run_node), target, category)

            intree = (self.top_node == self.run_node)
            if intree:
                for category, name, nodes, from_node, target_dir in self.outputs_registry.iter_over_category():
                    for node in nodes:
                        if node != target_node and node.is_bld():
                            _install_node(category, node, from_node, target_dir)
            else:
                for category, name, nodes, from_node, target_dir in self.outputs_registry.iter_over_category():
                    for node in nodes:
                        if node != target_node:
                            _install_node(category, node, from_node, target_dir)

class SdistContext(CmdContext):
    def __init__(self, global_context, cmd_args, option_context, pkg, run_node):
        super(SdistContext, self).__init__(global_context, cmd_args, option_context, pkg, run_node)
        self._meta = {}

        self._node_pkg = NodeRepresentation(run_node, self.top_node)
        self._node_pkg.update_package(pkg)

    def register_metadata(self, name, value):
        self._meta[name] = value

    def register_source_node(self, node, archive_name=None):
        """Register a node into the source distribution.

        archive_name is an optional string which will be used for the file name
        in the archive."""
        self._node_pkg._extra_source_nodes.append(node)
        if archive_name:
            self._node_pkg._aliased_source_nodes[node] = archive_name

    def configure(self):
        if self.pkg.meta_template_files:
            for template in self.pkg.meta_template_files:
                output = write_template(self.top_node, template, self.pkg, self._meta)
                self.register_source_node(output, output.bldpath())

class HelpContext(CmdContext):
    def __init__(self, *a, **kw):
        super(HelpContext, self).__init__(*a, **kw)
        self.short_descriptions = {}
        for cmd_name in self._global_context.command_names(public_only=False):
            cmd = self._global_context.retrieve_command(cmd_name)
            self.short_descriptions[cmd_name] = cmd.short_descr

    def retrieve_options_context(self, cmd_name):
        return self._global_context.retrieve_options_context(cmd_name)

    def is_options_context_registered(self, cmd_name):
        return self._global_context.is_options_context_registered(cmd_name)



import sys
import os
if sys.platform == "win32":
    import _winreg

import yaku.utils
import yaku.task

if sys.platform == "win32":
    from yaku.tools.mscommon.common \
        import \
            read_keys, open_key, close_key, get_output
    from yaku.tools.msvc \
        import \
            _exec_command_factory

_ROOT = {"amd64": r"Software\Wow6432Node\Intel\Suites",
         "ia32": r"Software\Intel\Compilers"}

_FC_ROOT = {"amd64": r"Software\Wow6432Node\Intel\Compilers",
         "ia32": r"Software\Intel\Compilers"}

_ABI2BATABI = {"amd64": "intel64", "ia32": "ia32"}

def find_versions_fc(abi):
    base = _winreg.HKEY_LOCAL_MACHINE
    key = os.path.join(_FC_ROOT[abi], "Fortran")

    availables = {}
    versions = read_keys(base, key)
    if versions is None:
        return availables
    for v in versions:
        verk = os.path.join(key, v)
        key = open_key(verk)
        try:
            maj = _winreg.QueryValueEx(key, "Major Version")[0]
            min = _winreg.QueryValueEx(key, "Minor Version")[0]
            bld = _winreg.QueryValueEx(key, "Revision")[0]
            availables[(maj, min, bld)] = verk
        finally:
            close_key(key)
    return availables

def product_dir_fc(root):
    k = open_key(root)
    try:
        return _winreg.QueryValueEx(k, "ProductDir")[0]
    finally:
        close_key(k)

def setup(ctx):
    env = ctx.env

    env.update(
       {"F77": ["ifort"],
        "F77_LINK": ["ifort"],
        "F77_LINKFLAGS": [],
        "F77FLAGS": [],
        "F77_TGT_F": ["-o"],
        "F77_SRC_F": ["-c"],
        "F77_LINK_TGT_F": ["-o"],
        "F77_LINK_SRC_F": [],
        "F77_OBJECT_FMT": "%s.o",
        "F77_PROGRAM_FMT": "%s"})
    if sys.platform == "win32":
        env.update(
           {"F77FLAGS": ["/nologo"],
            "F77_TGT_F": ["/object:"],
            "F77_SRC_F": ["/c"],
            "F77_LINKFLAGS": ["/nologo"],
            "F77_LINK_TGT_F": ["/link", "/out:"],
            "F77_OBJECT_FMT": "%s.obj",
            "F77_PROGRAM_FMT": "%s.exe"})

        abi = "amd64"

        availables = find_versions_fc(abi)
        if len(availables) < 1:
            raise ValueError("No ifort version found for abi %s" % abi)

        versions = sorted(availables.keys())[::-1]
        pdir = product_dir_fc(availables[versions[0]])
        batfile = os.path.join(pdir, "bin", "ifortvars.bat")

        d = get_output(ctx, batfile, _ABI2BATABI[abi])
        for k, v in d.items():
            if k in ["LIB"]:
                ctx.env.extend("LIBDIR", v, create=True)
            elif k in ["INCLUDE"]:
                ctx.env.extend("CPPPATH", v, create=True)
        for p in d["PATH"]:
            exe = os.path.join(p, "ifort.exe")
            if os.path.exists(exe):
                env["F77"] = [exe]
                env["F77_LINK"] = [exe]
                break

if sys.platform == "win32":
    for task_class in ["f77", "fprogram"]:
       klass = yaku.task.task_factory(task_class)
       saved = klass.exec_command
       klass.exec_command = _exec_command_factory(saved)

def detect(ctx):
    if yaku.utils.find_program("ifort") is None:
        return False
    else:
        return True



import os
import sys
import subprocess

import six

import plistlib

from bento.compat.api \
    import \
        check_call

BENTO_INFO = "0.0.5"

MKBOM = "/usr/bin/mkbom"

def _unicode(*args, **kw):
    if six.PY3:
        return str(*args, **kw)
    else:
        return unicode(*args, **kw)

def path_requirement(SpecArgument, Level=six.u('requires'), **kw):
    return dict(
        Level=Level,
        SpecType=six.u('file'),
        SpecArgument=unicode_path(SpecArgument),
        SpecProperty=six.u('NSFileType'),
        TestOperator=six.u('eq'),
        TestObject=six.u('NSFileTypeDirectory'),
        **kw
    )

def common_info(pkg_info):
    # Keys that can appear in any package
    name = _unicode(pkg_info.name)
    major, minor = pkg_info.version_info[0], pkg_info.version_info[1]
    version = pkg_info.version
    defaults = dict(
        CFBundleGetInfoString='%s %s' % (name, version),
        CFBundleIdentifier='org.pythonmac.%s' % (name,),
        CFBundleName=name,
        CFBundleShortVersionString=_unicode(version),
        IFMajorVersion=major,
        IFMinorRevision=minor,
        IFPkgFormatVersion=0.10000000149011612,
        IFRequirementDicts=[path_requirement(six.u('/'))],
        PythonInfoDict=dict(
            PythonLongVersion=_unicode(sys.version),
            PythonShortVersion=_unicode(sys.version[:3]),
            PythonExecutable=_unicode(sys.executable),
            bento_version=dict(
                version=BENTO_INFO
            ),
        ),
    )
    return defaults

def common_description(pkg_info):
    return dict(
        IFPkgDescriptionTitle=_unicode(pkg_info.name),
        IFPkgDescriptionVersion=_unicode(pkg_info.version),
    )

def unicode_path(path, encoding=sys.getfilesystemencoding()):
    if isinstance(path, six.text_type):
        return path
    return _unicode(path, encoding)

def write(dct, path):
    p = plistlib.Plist()
    p.update(dct)
    p.write(path)

def ensure_directories(pkg_info):
    for d in [pkg_info.contents, pkg_info.resources, pkg_info.en_lproj]:
        if not os.path.exists(d):
            os.makedirs(d)

def build_bom(pkg_info):
    check_call([MKBOM, pkg_info.source_root, pkg_info.bom])

def build_archive(pkg_info):
    check_call(["pax", "-w", "-f", pkg_info.archive, "-x", "cpio", "-z", "."],
                          cwd=pkg_info.source_root)

def build_info_plist(pkg_info):
    d = common_info(pkg_info)

    # Keys that can only appear in single packages
    d.update(dict(
        IFPkgFlagAllowBackRev=False,
        IFPkgFlagAuthorizationAction=six.u('AdminAuthorization'),
        IFPkgFlagFollowLinks=True,
        IFPkgFlagInstallFat=False,
        IFPkgFlagIsRequired=False,
        IFPkgFlagOverwritePermissions=False,
        IFPkgFlagRelocatable=False,
        IFPkgFlagRestartAction=six.u('NoRestart'),
        IFPkgFlagRootVolumeOnly=True,
        IFPkgFlagUpdateInstalledLangauges=False,
    ))

    d.update(dict(
        IFPkgFlagAuthorizationAction=pkg_info.auth,
        IFPkgFlagDefaultLocation=unicode_path(pkg_info.prefix),
    ))
    write(d, pkg_info.info_plist)

def build_pkg_info(pkg_info):
    fid = open(pkg_info.pkg_info, "w")
    try:
        fid.write("pmkrpkg1")
    finally:
        fid.close()

def build_description_plist(pkg_info):
    desc = common_description(pkg_info)
    desc['IFPkgDescriptionDescription'] = pkg_info.description
    write(desc, pkg_info.description_plist)

def build_pkg(pkg_info):
    ensure_directories(pkg_info)

    build_bom(pkg_info)
    build_archive(pkg_info)

    build_info_plist(pkg_info)

    build_pkg_info(pkg_info)

    build_description_plist(pkg_info)

class PackageInfo(object):
    def __init__(self, pkg_name, prefix, source_root, pkg_root, admin=True, description=None, version=None):
        if admin:
            self.auth = six.u("AdminAuthorization")
        else:
            self.auth = six.u("RootAuthorization")

        # Where things will be installed by Mac OS X installer
        self.prefix = prefix
        # Root directory for files to be packaged
        self.source_root = source_root
        # Root directory for produced .pkg directory/file
        self.pkg_root = pkg_root

        self.name = pkg_name
        # FIXME: version handling -> use distutils2 version module
        self.version_info = (0, 0, 5, None)
        if version is None:
            self.version = ""
        else:
            self.version = version

        if description:
            self.description = description
        else:
            self.description = ""

        self.contents = os.path.join(self.pkg_root, "Contents")
        self.resources = os.path.join(self.contents, "Resources")
        self.en_lproj = os.path.join(self.resources, "en.lproj")

        self.bom = os.path.join(self.contents, "Archive.bom")
        self.archive = os.path.join(self.contents, "Archive.pax.gz")
        self.info_plist = os.path.join(self.contents, "Info.plist")
        self.pkg_info = os.path.join(self.contents, "PkgInfo")

        self.description_plist = os.path.join(self.en_lproj, "Description.plist")

class MetaPackageInfo(object):
    @classmethod
    def from_ipkg(cls, ipkg):
        m = ipkg.meta
        info_string = "%s %s" % (m["name"], m["version"])
        identifier = "com.github.cournape.bento"
        version_info = (0, 0, 5)
        return cls(m["name"], info_string, version_info, identifier, m["summary"])

    def __init__(self, name, info_string, version_info, identifier, summary):
        self.major, self.minor, self.micro = version_info[0], version_info[1], version_info[2]

        self.info_string = info_string
        self.name = name
        self.identifier = identifier
        self.description = summary

        self.short_version = ".".join([str(i) for i in [self.major, self.minor, self.micro]])

def make_mpkg_plist(mpkg_info, path):
    pl = dict(
            CFBundleGetInfoString=mpkg_info.info_string,
            CFBundleIdentifier=mpkg_info.identifier,
            CFBundleName=mpkg_info.name,
            CFBundleShortVersionString=mpkg_info.short_version,
            IFMajorVersion=mpkg_info.major,
            IFMinorVersion=mpkg_info.minor,
            IFPkgFlagComponentDirectory="Contents/Packages",
            IFPkgFlagPackageList=[
                dict(
                    IFPkgFlagPackageLocation=pkg,
                    IFPkgFlagPackageSelection='selected'
                )
                for pkg in mpkg_info.packages
            ],
            IFPkgFormatVersion=0.10000000149011612,
            IFPkgFlagBackgroundScaling="proportional",
            IFPkgFlagBackgroundAlignment="left",
            IFPkgFlagAuthorizationAction="RootAuthorization",
        )

    write(pl, path)
    return pl

def make_mpkg_description(mpkg_info, path):
    d = dict(IFPkgDescriptionTitle=mpkg_info.name,
             IFPkgDescriptionDescription=mpkg_info.description,
             IFPkgDescriptionVersion=mpkg_info.short_version)
    write(d, path)



import os

from bento.distutils.utils \
    import \
        _is_setuptools_activated
if _is_setuptools_activated():
    from setuptools \
        import \
            Distribution
else:
    from distutils.dist \
        import \
            Distribution

from bento.commands.configure \
    import \
        _setup_options_parser
from bento.commands.core \
    import \
        HelpCommand
from bento.commands.hooks \
    import \
        find_pre_hooks, find_post_hooks, find_startup_hooks, \
        find_shutdown_hooks, find_options_hooks, find_command_hooks
from bento.commands.wrapper_utils \
    import \
        set_main
from bento.compat.api \
    import \
        defaultdict
from bento.conv \
    import \
        pkg_to_distutils_meta
from bento.core.node \
    import \
        create_root_with_source_tree
from bento.core.package \
    import \
        PackageDescription
from bento.core.options \
    import \
        PackageOptions
from bento.commands.contexts \
    import \
        GlobalContext
from bento.commands.command_contexts \
    import \
        CmdContext, SdistContext, ContextWithBuildDirectory
from bento.commands.registries \
    import \
        CommandRegistry, ContextRegistry, OptionsRegistry
from bento.backends.yaku_backend \
    import \
        ConfigureYakuContext, BuildYakuContext
from bento.commands.build_egg \
    import \
        BuildEggCommand
import bento.commands.wrapper_utils

from bento.commands.dependency \
    import \
        CommandScheduler
from bento.commands.build \
    import \
        BuildCommand
from bento.commands.configure \
    import \
        ConfigureCommand
from bento.commands.install \
    import \
        InstallCommand
from bento.commands.sdist \
    import \
        SdistCommand
from bento.commands.options \
    import \
        OptionsContext
from bento.distutils.commands.config \
    import \
        config
from bento.distutils.commands.build \
    import \
        build
from bento.distutils.commands.install \
    import \
        install
from bento.distutils.commands.sdist \
    import \
        sdist

_BENTO_MONKEYED_CLASSES = {"build": build, "config": config, "install": install, "sdist": sdist}

if _is_setuptools_activated():
    from bento.distutils.commands.bdist_egg \
        import \
            bdist_egg
    from bento.distutils.commands.egg_info \
        import \
            egg_info
    _BENTO_MONKEYED_CLASSES["bdist_egg"] = bdist_egg
    _BENTO_MONKEYED_CLASSES["egg_info"] = egg_info

def _setup_cmd_classes(attrs):
    cmdclass = attrs.get("cmdclass", {})
    for klass in _BENTO_MONKEYED_CLASSES:
        if not klass in cmdclass:
            cmdclass[klass] = _BENTO_MONKEYED_CLASSES[klass]
    attrs["cmdclass"] = cmdclass
    return attrs

def global_context_factory(package_options):
    # FIXME: factor this out with the similar code in bentomakerlib
    global_context = GlobalContext(None)
    global_context.register_package_options(package_options)

    register_commands(global_context)
    register_command_contexts(global_context)
    for cmd_name in global_context.command_names():
        cmd = global_context.retrieve_command(cmd_name)
        options_context = OptionsContext.from_command(cmd)

        if not global_context.is_options_context_registered(cmd_name):
            global_context.register_options_context(cmd_name, options_context)

    return global_context

def register_command_contexts(global_context):
    default_mapping = defaultdict(lambda: ContextWithBuildDirectory)
    default_mapping.update(dict([
            ("configure", ConfigureYakuContext),
            ("build", BuildYakuContext),
            ("install", ContextWithBuildDirectory),
            ("sdist", SdistContext)]))

    for cmd_name in global_context.command_names(public_only=False):
        if not global_context.is_command_context_registered(cmd_name):
            global_context.register_command_context(cmd_name, default_mapping[cmd_name])

def register_commands(global_context):
    global_context.register_command("configure", ConfigureCommand())
    global_context.register_command("build", BuildCommand())
    global_context.register_command("install", InstallCommand())
    global_context.register_command("sdist", SdistCommand())
    global_context.register_command("build_egg", BuildEggCommand())

class BentoDistribution(Distribution):
    def get_command_class(self, command):
        # Better raising an error than having some weird behavior for a command
        # we don't support
        if self.script_args is not None \
           and command in self.script_args \
           and command not in _BENTO_MONKEYED_CLASSES:
            raise ValueError("Command %s is not supported by bento.distutils compat layer" % command)
        return Distribution.get_command_class(self, command)

    def __init__(self, attrs=None):
        if attrs is None:
            attrs = {}

        if not "bento_info" in attrs:
            bento_info = "bento.info"
        else:
            bento_info = attrs["bento.info"]
        self.pkg = PackageDescription.from_file(bento_info)
        package_options = PackageOptions.from_file(bento_info)

        attrs = _setup_cmd_classes(attrs)

        d = pkg_to_distutils_meta(self.pkg)
        attrs.update(d)

        Distribution.__init__(self, attrs)

        self.packages = self.pkg.packages
        self.py_modules = self.pkg.py_modules
        if hasattr(self, "entry_points"):
            if self.entry_points is None:
                self.entry_points = {}
            console_scripts = [e.full_representation() for e in self.pkg.executables.values()]
            if "console_scripts" in self.entry_points:
                self.entry_points["console_scripts"].extend(console_scripts)
            else:
                self.entry_points["console_scripts"] = console_scripts

        source_root = os.getcwd()
        build_root = os.path.join(source_root, "build")
        root = create_root_with_source_tree(source_root, build_root)
        self.top_node = root._ctx.srcnode
        self.build_node = root._ctx.bldnode
        self.run_node = root._ctx.srcnode

        self.global_context = global_context_factory(package_options)
        mods = set_main(self.pkg, self.top_node, self.build_node)
        self._setup_hooks(self.pkg, self.global_context, mods)

    def _setup_hooks(self, package, global_context, mods):
        if package.use_backends:
            if len(package.use_backends) > 1:
                raise ValueError("Only up to one backend supported for now")
            else:
                assert global_context.backend is None
                global_context.backend = load_backend(package.use_backends[0])()

        startup_hooks = find_startup_hooks(mods)
        option_hooks = find_options_hooks(mods)
        shutdown_hooks = find_shutdown_hooks(mods)

        if startup_hooks:
            # FIXME: there should be an error or a warning if startup defined in
            # mods beyond the first one
            startup_hooks[0](global_context)

        if global_context.backend:
            global_context.backend.register_command_contexts(global_context)
        for command in find_command_hooks(mods):
            global_context.register_command(command.name, command)

        if global_context.backend:
            global_context.backend.register_options_contexts(global_context)

        if option_hooks:
            # FIXME: there should be an error or a warning if shutdown defined in
            # mods beyond the first one
            option_hooks[0](global_context)

        # FIXME: this registered options for new commands registered in hook. It
        # should be made all in one place (hook and non-hook)
        for cmd_name in global_context.command_names(public_only=False):
            if not global_context.is_options_context_registered(cmd_name):
                # FIXME: this should be supported in global context directly
                # (redundant with bentomakerlib)
                cmd = global_context.retrieve_command(cmd_name)
                context = OptionsContext.from_command(cmd)

                if not global_context.is_options_context_registered(cmd_name):
                    global_context.register_options_context(cmd_name, context)

        for cmd_name in global_context.command_names():
            for hook in find_pre_hooks(mods, cmd_name):
                global_context.add_pre_hook(hook, cmd_name)
            for hook in find_post_hooks(mods, cmd_name):
                global_context.add_post_hook(hook, cmd_name)

    def run_command_in_context(self, cmd_name, cmd_argv):
        return bento.commands.wrapper_utils.resolve_and_run_command(self.global_context,
            cmd_name, cmd_argv, self.run_node, self.top_node, self.pkg)

    def has_data_files(self):
        return len(self.pkg.data_files) > 0        



import base64
import sys

import os.path as op

from bento.pypi.register_utils \
    import \
        encode_multipart, _BOUNDARY
from bento.conv \
    import \
        pkg_to_distutils_meta_pkg_info
from bento.utils.utils \
    import \
        extract_exception
from bento.errors \
    import \
        PyPIError, InvalidRepository

from six \
    import \
        PY3

import six

try:
    from hashlib import md5
except ImportError:
    from md5 import md5




if PY3:
    from urllib.request \
        import \
            Request, HTTPBasicAuthHandler, HTTPError, URLError, urlparse, urlopen
else:
    from urllib2 \
        import \
            Request, HTTPBasicAuthHandler, HTTPError, URLError, urlopen
    from urlparse \
        import \
            urlparse

def build_upload_post_data(filename, dist_type, package, sign=False, comment=""):
    pyversion = ".".join(str(i) for i in sys.version_info[:2])

    f = open(filename,'rb')
    try:
        content = f.read()
    finally:
        f.close()

    data = pkg_to_distutils_meta_pkg_info(package)
    data[":action"] = "file_upload"
    data["protocol_version"] = "1"
    data.update({
        # file content
        'content': (op.basename(filename), content),
        'filetype': dist_type,
        'pyversion': pyversion,
        'md5_digest': md5(content).hexdigest(),

        # additional meta-data
        'metadata_version' : '1.0',

        'comment': comment,
    })

    if sign:
        raise NotImplementedError("Signing not yet implemented.")
        data['gpg_signature'] = (op.basename(filename) + ".asc",
                                 open(filename+".asc").read())

    return data

def build_request(repository, post_data, auth):
    files = []
    for key in ('content', 'gpg_signature'):
        if key in post_data:
            filename_, value = post_data.pop(key)
            files.append((key, filename_, value))
    content_type, body = encode_multipart(post_data.items(), files)

    headers = {'Content-type': content_type,
               'Content-length': str(len(body)),
               'Authorization': auth}

    return Request(repository, data=body, headers=headers)

def upload(dist_filename, dist_type, package, config, sign=False):
    schema, netloc, url, params, query, fragments = urlparse(config.repository)
    if params or query or fragments:
        raise InvalidRepository("Incompatible url %s" % config.repository)

    if schema not in ('http', 'https'):
        raise InvalidRepository("unsupported schema " + schema)

    if sign:
        raise NotImplementedError()

    data = build_upload_post_data(dist_filename, dist_type, package)
    userpass = (config.username + ":" + config.password).encode("ascii")
    auth = six.b("Basic ") + base64.standard_b64encode(userpass)
    request = build_request(config.repository, data, auth)

    try:
        result = urlopen(request)
        status = result.getcode()
        reason = result.msg
    except HTTPError:
        e = extract_exception()
        status = e.code
        reason = e.msg

    if status != 200:
        raise PyPIError(
                "Could not upload to repository %r - error %s (server answered '%s')" \
                % (config.repository, status, reason))



import os
import sys
import shutil
import tempfile

import os.path as op

from six.moves \
    import \
        cStringIO

from bento.compat.api.moves \
    import \
        unittest


from bento.core.options \
    import \
        PackageOptions
from bento.core.package \
    import \
        PackageDescription
from bento.core.testing\
    import \
        expected_failure, skip_if
from bento.core.node \
    import \
        create_base_nodes
from bento.utils.utils \
    import \
        subst_vars
from bento.backends.distutils_backend \
    import \
        DistutilsBuildContext, DistutilsConfigureContext
from bento.backends.yaku_backend \
    import \
        BuildYakuContext, ConfigureYakuContext, YakuBackend
from bento.commands.hooks \
    import \
        PreHookWrapper
from bento.commands.wrapper_utils \
    import \
        run_command_in_context
from bento.commands.options \
    import \
        OptionsContext
from bento.commands.build \
    import \
        BuildCommand
from bento.core.testing \
    import \
        create_fake_package_from_bento_infos, create_fake_package_from_bento_info, \
        require_c_compiler
from bento.commands.tests.utils \
    import \
        prepare_configure, prepare_build, EncodedStringIO, \
        prepare_command, create_global_context, prepare_package
from bento.errors \
    import \
        UsageException


BENTO_INFO_WITH_EXT = """\
Name: foo

Library:
    Extension: foo
        Sources: foo.c
"""

BENTO_INFO_WITH_CLIB = """\
Name: foo

Library:
    CompiledLibrary: foo
        Sources: foo.c
"""

BENTO_INFO = """\
Name: foo

Library:
    Packages: foo, foo.bar
    Modules: fubar
"""

class _TestBuildSimpleExtension(unittest.TestCase):
    # Those should be set by subclasses
    _configure_context = None
    _build_context = None
    def setUp(self):
        self.save = None
        self.d = None

        self.save = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)

        try:
            self.top_node, self.build_node, self.run_node = \
                    create_base_nodes(self.d, os.path.join(self.d, "build"))

            def builder_factory(build_context):
                def _dummy(extension):
                    from_node = self.build_node

                    pkg_dir = op.dirname(extension.name.replace('.', op.sep))
                    target_dir = op.join('$sitedir', pkg_dir)
                    build_context.outputs_registry.register_outputs("extensions",
                        extension.name, [], from_node, target_dir)
                return _dummy

            self._builder_factory = builder_factory
        except:
            os.chdir(self.save)
            raise

    def tearDown(self):
        if self.save:
            os.chdir(self.save)
        if self.d:
            shutil.rmtree(self.d)

    def _run_configure_and_build(self, bentos, bscripts=None, configure_argv=None, build_argv=None):
        conf, configure, bld, build = self._run_configure(bentos, bscripts,
                configure_argv, build_argv)
        run_command_in_context(bld, build)

        return conf, configure, bld, build

    def _run_configure(self, bentos, bscripts=None, configure_argv=None, build_argv=None):
        top_node = self.top_node

        create_fake_package_from_bento_infos(top_node, bentos, bscripts)

        conf, configure = prepare_configure(top_node, bentos["bento.info"], self._configure_context)
        run_command_in_context(conf, configure)

        bld, build = prepare_build(top_node, bentos["bento.info"], self._build_context, build_argv)

        return conf, configure, bld, build

    def _resolve_isection(self, node, isection):
        source_dir = subst_vars(isection.source_dir, {"_srcrootdir": self.build_node.abspath()})
        isection.source_dir = source_dir
        return isection

    def test_no_extension(self):
        self._run_configure_and_build({"bento.info": BENTO_INFO})

    def test_simple_extension(self):
        conf, configure, bld, build = self._run_configure_and_build({"bento.info": BENTO_INFO_WITH_EXT})

        sections = bld.section_writer.sections["extensions"]
        for extension in conf.pkg.extensions.values():
            isection = self._resolve_isection(bld.run_node, sections[extension.name])
            self.assertTrue(os.path.exists(os.path.join(isection.source_dir, isection.files[0][0])))

    def test_disable_extension(self):
        conf, configure, bld, build = self._run_configure({"bento.info": BENTO_INFO_WITH_EXT})
        pre_hook = PreHookWrapper(lambda context: context.disable_extension("foo"),
                                  "build", self.d)
        run_command_in_context(bld, build, pre_hooks=[pre_hook])

        assert "extensions" not in bld.section_writer.sections

    @expected_failure
    def test_disable_nonexisting_extension(self):
        conf, configure, bld, build = self._run_configure({"bento.info": BENTO_INFO_WITH_EXT})

        pre_hook = PreHookWrapper(lambda context: context.disable_extension("foo2"),
                                  "build", self.d)
        run_command_in_context(bld, build, pre_hooks=[pre_hook])

        assert "extensions" not in bld.section_writer.sections

    def test_extension_registration(self):
        top_node = self.top_node

        bento_info = """\
Name: foo

Library:
    Extension: _foo
        Sources: src/foo.c
    Extension: _bar
        Sources: src/bar.c
"""
        bentos = {"bento.info": bento_info}

        conf, configure, bld, build = self._run_configure(bentos)

        def pre_build(context):
            builder = self._builder_factory(context)
            context.register_builder("_bar", builder)
        pre_hook = PreHookWrapper(pre_build, "build", self.d)
        run_command_in_context(bld, build, pre_hooks=[pre_hook])

        sections = bld.section_writer.sections["extensions"]
        self.failUnless(len(sections) == 2)
        self.failUnless(len(sections["_bar"].files) == 0)

    def test_extension_tweak(self):
        bento_info = """\
Name: foo

Library:
    Extension: _foo
        Sources: src/foo.c
    Extension: _bar
        Sources: src/bar.c
"""
        bentos = {"bento.info": bento_info}

        conf, configure, bld, build = self._run_configure(bentos)

        # To check that include_dirs is passed along the slightly
        # over-engineered callchain, we wrap the default build with a function
        # that check the include_dirs is step up correctly.
        def pre_build(context):
            context.tweak_extension("_bar", include_dirs=["fubar"])
        pre_hook = PreHookWrapper(pre_build, "build", self.d)
        old_default_builder = bld.default_builder
        try:
            def _builder_wrapper(extension, **kw):
                if extension.name == "_bar":
                    self.assertTrue(kw.get("include_dirs", []) == ["fubar"])
                return old_default_builder(extension, **kw)
            bld.default_builder = _builder_wrapper
            run_command_in_context(bld, build, pre_hooks=[pre_hook])
        finally:
            bld.default_builder = old_default_builder

        sections = bld.section_writer.sections["extensions"]
        self.failUnless(len(sections) == 2)

    def test_simple_library(self):
        conf, configure, bld, build = self._run_configure_and_build({"bento.info": BENTO_INFO_WITH_CLIB})
        sections = bld.section_writer.sections["compiled_libraries"]
        for library in conf.pkg.compiled_libraries.values():
            isection = self._resolve_isection(bld.run_node, sections[library.name])
            self.assertTrue(os.path.exists(os.path.join(isection.source_dir, isection.files[0][0])))

    def test_simple_inplace(self):
        _, _, bld, build = self._run_configure_and_build({"bento.info": BENTO_INFO_WITH_EXT}, build_argv=["-i"])
        sections = bld.section_writer.sections["extensions"]
        source_dir = bld.top_node.abspath()
        for section in sections.values():
            isection = self._resolve_isection(bld.run_node, section)
            for f in isection.files[0]:
                self.assertTrue(op.exists(op.join(source_dir, f)))

class TestBuildDistutils(_TestBuildSimpleExtension):
    def setUp(self):
        from bento.commands.build_distutils import DistutilsBuilder
        _TestBuildSimpleExtension.setUp(self)
        self._distutils_builder = DistutilsBuilder()

        self._configure_context = DistutilsConfigureContext
        self._build_context = DistutilsBuildContext

    @require_c_compiler("distutils")
    def test_simple_inplace(self):
        super(TestBuildDistutils, self).test_simple_inplace()

    @require_c_compiler("distutils")
    def test_extension_registration(self):
        super(TestBuildDistutils, self).test_extension_registration()

    @require_c_compiler("distutils")
    def test_simple_library(self):
        super(TestBuildDistutils, self).test_simple_library()

    @require_c_compiler("distutils")
    def test_simple_extension(self):
        super(TestBuildDistutils, self).test_simple_extension()

class TestBuildYaku(_TestBuildSimpleExtension):
    def setUp(self):
        super(TestBuildYaku, self).setUp()

        self._configure_context = ConfigureYakuContext
        self._build_context = BuildYakuContext

    @require_c_compiler("yaku")
    def test_simple_inplace(self):
        super(TestBuildYaku, self).test_simple_inplace()

    @require_c_compiler("yaku")
    def test_extension_registration(self):
        super(TestBuildYaku, self).test_extension_registration()

    @require_c_compiler("yaku")
    def test_simple_library(self):
        super(TestBuildYaku, self).test_simple_library()

    @require_c_compiler("yaku")
    def test_simple_extension(self):
        super(TestBuildYaku, self).test_simple_extension()

    @require_c_compiler("yaku")
    def test_disable_extension(self):
        super(TestBuildYaku, self).test_disable_extension()

    @require_c_compiler("yaku")
    def test_disable_nonexisting_extension(self):
        super(TestBuildYaku, self).test_disable_nonexisting_extension()

def _not_has_waf():
    try:
        import bento.backends.waf_backend
        bento.backends.waf_backend.disable_output()
        return False
    except SyntaxError:
        return True
    except UsageException:
        return True

def skip_no_waf(f):
    return skip_if(_not_has_waf(), "waf not found")(f)

class TestBuildWaf(_TestBuildSimpleExtension):
    #def __init__(self, *a, **kw):
    #    super(TestBuildWaf, self).__init__(*a, **kw)

    #    # Add skip_no_waf decorator to any test function
    #    for m in dir(self):
    #        a = getattr(self, m)
    #        if isinstance(a, types.MethodType) and _NOSE_CONFIG.testMatch.match(m):
    #            setattr(self, m, skip_no_waf(a))

    def _run_configure(self, bentos, bscripts=None, configure_argv=None, build_argv=None):
        from bento.backends.waf_backend import make_stream_logger
        from bento.backends.waf_backend import WafBackend

        top_node = self.top_node

        bento_info = bentos["bento.info"]
        package = PackageDescription.from_string(bento_info)
        package_options = PackageOptions.from_string(bento_info)

        create_fake_package_from_bento_info(top_node, bento_info)
        top_node.make_node("bento.info").safe_write(bento_info)

        global_context = create_global_context(package, package_options, WafBackend())
        conf, configure = prepare_command(global_context, "configure",
                configure_argv, package, top_node)
        run_command_in_context(conf, configure)

        bld, build = prepare_command(global_context, "build", build_argv, package, top_node)
        bld.waf_context.logger = make_stream_logger("build", cStringIO())
        return conf, configure, bld, build

    @skip_no_waf
    def test_simple_extension(self):
        super(TestBuildWaf, self).test_simple_extension()

    @skip_no_waf
    def test_simple_library(self):
        super(TestBuildWaf, self).test_simple_library()

    @skip_no_waf
    def test_no_extension(self):
        super(TestBuildWaf, self).test_no_extension()

    @skip_no_waf
    def test_extension_registration(self):
        super(TestBuildWaf, self).test_extension_registration()

    @skip_no_waf
    def test_disable_extension(self):
        super(TestBuildWaf, self).test_disable_extension()

    @skip_no_waf
    def test_disable_nonexisting_extension(self):
        super(TestBuildWaf, self).test_disable_extension()

    @skip_no_waf
    def test_simple_inplace(self):
        super(TestBuildWaf, self).test_simple_inplace()

    @skip_no_waf
    def test_extension_tweak(self):
        super(TestBuildWaf, self).test_extension_tweak()

    def setUp(self):
        self._fake_output = None
        self._stderr = sys.stderr
        self._stdout = sys.stdout
        super(TestBuildWaf, self).setUp()
        # XXX: ugly stuff to make waf and nose happy together
        sys.stdout = EncodedStringIO()
        sys.stderr = EncodedStringIO()

        if _not_has_waf():
            return
        else:
            from bento.backends.waf_backend import ConfigureWafContext, BuildWafContext

            self._configure_context = ConfigureWafContext
            self._build_context = BuildWafContext

    def tearDown(self):
        super(TestBuildWaf, self).tearDown()
        sys.stderr = self._stderr
        sys.stdout = self._stdout

class TestBuildCommand(unittest.TestCase):
    def setUp(self):
        self.d = tempfile.mkdtemp()
        self.top_node, self.build_node, self.run_node = create_base_nodes(self.d, os.path.join(self.d, "build"))

        self.old_dir = os.getcwd()
        os.chdir(self.d)

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

    def _execute_build(self, bento_info):
        create_fake_package_from_bento_info(self.top_node, bento_info)
        # FIXME: this should be done automatically in create_fake_package_from_bento_info
        self.top_node.make_node("bento.info").safe_write(bento_info)

        package = PackageDescription.from_string(bento_info)
        package_options = PackageOptions.from_string(bento_info)

        global_context = create_global_context(package, package_options, YakuBackend())
        conf, configure = prepare_command(global_context, "configure", [], package, self.run_node)
        run_command_in_context(conf, configure)

        bld, build = prepare_command(global_context, "build", [], package, self.run_node)
        run_command_in_context(bld, build)

        return bld

    def test_simple(self):
        self._execute_build(BENTO_INFO)

    def test_executables(self):
        bento_info = """\
Name: foo

Library:
    Packages: foo

Executable: foomaker
    Module: foomain
    Function: main
"""
        self._execute_build(bento_info)

    def test_config_py(self):
        bento_info = """\
Name: foo

ConfigPy: foo/__config.py

Library:
    Packages: foo
"""
        self._execute_build(bento_info)

    def test_meta_template_file(self):
        bento_info = """\
Name: foo

MetaTemplateFiles: foo/__package_info.py.in

Library:
    Packages: foo
"""
        n = self.top_node.make_node(op.join("foo", "__package_info.py.in"))
        n.parent.mkdir()
        n.write("""\
NAME = $NAME
""")
        self._execute_build(bento_info)
        package_info = self.build_node.find_node(op.join("foo", "__package_info.py"))

        r_package_info = """\
NAME = "foo"
"""
        self.assertEqual(package_info.read(), r_package_info)

class TestBuildDirectoryBase(unittest.TestCase):
    def setUp(self):
        self.d = tempfile.mkdtemp()

        try:
            self.top_node, self.build_node, self.run_node = create_base_nodes(self.d,
                    os.path.join(self.d, "yoyobuild"))

            self.old_dir = os.getcwd()
            os.chdir(self.d)
        except:
            shutil.rmtree(self.d)
            raise

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

class TestBuildDirectory(TestBuildDirectoryBase):
    def test_simple_yaku(self):
        top_node = self.top_node

        create_fake_package_from_bento_info(top_node, BENTO_INFO_WITH_EXT)
        conf, configure = prepare_configure(top_node, BENTO_INFO_WITH_EXT, ConfigureYakuContext)
        run_command_in_context(conf, configure)

        build = BuildCommand()
        opts = OptionsContext.from_command(build)

        bld = BuildYakuContext(None, [], opts, conf.pkg, top_node)
        run_command_in_context(bld, build)

    def test_simple_distutils(self):
        top_node = self.top_node

        create_fake_package_from_bento_info(top_node, BENTO_INFO_WITH_EXT)
        conf, configure = prepare_configure(top_node, BENTO_INFO_WITH_EXT, DistutilsConfigureContext)
        run_command_in_context(conf, configure)

        build = BuildCommand()
        opts = OptionsContext.from_command(build)

        bld = DistutilsBuildContext(None, [], opts, conf.pkg, top_node)
        run_command_in_context(bld, build)

class TestBuildDirectoryWaf(TestBuildDirectoryBase):
    def setUp(self):
        TestBuildDirectoryBase.setUp(self)

        self._stderr = sys.stderr
        self._stdout = sys.stdout

        sys.stdout = EncodedStringIO()
        sys.stderr = EncodedStringIO()

    def tearDown(self):
        sys.stderr = self._stderr
        sys.stdout = self._stdout

        TestBuildDirectoryBase.tearDown(self)

    @skip_no_waf
    def test_simple_waf(self):
        from bento.backends.waf_backend import make_stream_logger
        from bento.backends.waf_backend import WafBackend

        top_node = self.top_node

        package = PackageDescription.from_string(BENTO_INFO_WITH_EXT)
        package_options = PackageOptions.from_string(BENTO_INFO_WITH_EXT)

        create_fake_package_from_bento_info(top_node, BENTO_INFO_WITH_EXT)
        top_node.make_node("bento.info").safe_write(BENTO_INFO_WITH_EXT)

        global_context = create_global_context(package, package_options, WafBackend())
        conf, configure = prepare_command(global_context, "configure", [], package, top_node)
        run_command_in_context(conf, configure)

        bld, build = prepare_command(global_context, "build", [], package, top_node)
        bld.waf_context.logger = make_stream_logger("build", cStringIO())
        run_command_in_context(bld, build)

class _SandboxMixin(unittest.TestCase):
    def setUp(self):
        self.new_cwd = tempfile.mkdtemp()
        try:
            self.old_cwd = os.getcwd()
            os.chdir(self.new_cwd)
        except:
            shutil.rmtree(self.new_cwd)
            raise

    def tearDown(self):
        os.chdir(self.old_cwd)
        shutil.rmtree(self.new_cwd)

class TestOutputRegistration(_SandboxMixin):
    def setUp(self):
        super(TestOutputRegistration, self).setUp()
        self.top_node, self.build_node, self.run_node = create_base_nodes()

    def _run_build_with_pre_hook(self, hook_func):
        package = PackageDescription.from_string(BENTO_INFO)
        global_context = prepare_package(self.top_node, BENTO_INFO)

        conf, configure = prepare_command(global_context, "configure", [], package, self.top_node)
        run_command_in_context(conf, configure)

        pre_hook = PreHookWrapper(hook_func, self.build_node.path_from(self.top_node), self.top_node.abspath())
        bld, build = prepare_command(global_context, "build", [], package, self.top_node)
        run_command_in_context(bld, build, pre_hooks=[pre_hook])

        return bld

    def test_simple(self):
        def hook(context):
            context.register_category("dummy")
            n = context.make_build_node("foo.txt")
            context.register_outputs("dummy", "dummy1", [n])
        bld = self._run_build_with_pre_hook(hook)

        self.assertTrue("dummy" in bld.outputs_registry.categories)
        nodes = []
        for name, _nodes, source_dir, target_dir, in bld.outputs_registry.iter_category("dummy"):
            nodes.extend(_nodes)
        self.assertTrue(nodes, [self.build_node.find_node("foo.txt")])

    def test_simple_registration(self):
        def hook(context):
            n = context.make_build_node("foo.txt")
            context.register_outputs_simple([n])
            n = context.make_build_node("bar.txt")
            context.register_outputs_simple([n])
        bld = self._run_build_with_pre_hook(hook)

        self.assertTrue("hook_registered" in bld.outputs_registry.categories)
        nodes = []
        for name, _nodes, source_dir, target_dir, in bld.outputs_registry.iter_category("hook_registered"):
            nodes.extend(_nodes)
        self.assertTrue(nodes, [self.build_node.find_node("foo.txt")])



"""
Fortran-specific configuration tests
"""
import sys
import copy

from yaku.conftests.fconftests_imp \
    import \
        is_output_verbose, parse_flink

FC_VERBOSE_FLAG = "FC_VERBOSE_FLAG"
FC_RUNTIME_LDFLAGS = "FC_RUNTIME_LDFLAGS"
FC_DUMMY_MAIN = "FC_DUMMY_MAIN"

def check_fcompiler(conf, msg=None):
    code = """\
       program main
       end
"""
    if msg is None:
        conf.start_message("Checking whether Fortran compiler works")
    else:
        conf.start_message(msg)
    ret = conf.builders["fortran"].try_program("check_fcompiler", code)
    if ret:
        conf.end_message("yes")
    else:
        conf.end_message("no !")
        conf.fail_configuration("")
    return ret

def check_fortran_verbose_flag(conf):
    code = """\
       program main
       end
"""
    conf.start_message("Checking for verbose flag")
    if not conf.builders["ctasks"].configured:
        raise ValueError("'ctasks'r needs to be configured first!")
    if sys.platform == "win32":
        conf.end_message("none needed")
        conf.env[FC_VERBOSE_FLAG] = []
        return True
    for flag in ["-v", "--verbose", "-V", "-verbose"]:
        old = copy.deepcopy(conf.env["F77_LINKFLAGS"])
        try:
            conf.env["F77_LINKFLAGS"].append(flag)
            ret = conf.builders["fortran"].try_program("check_fc_verbose", code)
            if not ret:
                continue
            stdout = conf.get_stdout(conf.last_task)
            if ret and is_output_verbose(stdout):
                conf.end_message(flag)
                conf.env[FC_VERBOSE_FLAG] = flag
                return True
        finally:
            conf.env["F77_LINKFLAGS"] = old
    conf.end_message("failed !")
    conf.fail_configuration("")
    return False

def check_fortran_runtime_flags(conf):
    if not conf.builders["ctasks"].configured:
        raise ValueError("'ctasks'r needs to be configured first!")
    if sys.platform == "win32":
        return _check_fortran_runtime_flags_win32(conf)
    else:
        return _check_fortran_runtime_flags(conf)

def _check_fortran_runtime_flags_win32(conf):
    if conf.env["cc_type"] == "msvc":
        conf.start_message("Checking for fortran runtime flags")
        conf.end_message("none needed")
        conf.env[FC_RUNTIME_LDFLAGS] = []
    else:
        raise NotImplementedError("GNU support on win32 not ready")

def _check_fortran_runtime_flags(conf):
    if not FC_VERBOSE_FLAG in conf.env:
        raise ValueError("""\
You need to call check_fortran_verbose_flag before getting runtime
flags (or to define the %s variable)""" % FC_VERBOSE_FLAG)
    code = """\
       program main
       end
"""

    conf.start_message("Checking for fortran runtime flags")

    old = copy.deepcopy(conf.env["F77_LINKFLAGS"])
    try:
        conf.env["F77_LINKFLAGS"].append(conf.env["FC_VERBOSE_FLAG"])
        ret = conf.builders["fortran"].try_program("check_fc", code)
        if ret:
            stdout = conf.get_stdout(conf.last_task)
            flags = parse_flink(stdout)
            conf.end_message("%r" % " ".join(flags))
            conf.env[FC_RUNTIME_LDFLAGS] = flags
            return True
        else:
            conf.end_message("failed !")
            return False
    finally:
        conf.env["F77_LINKFLAGS"] = old
    return False

def check_fortran_dummy_main(conf):
    code_tpl = """\
#ifdef __cplusplus
        extern "C"
#endif
int %(main)s()
{
    return 1;
}

int main()
{
    return 0;
}
"""

    conf.start_message("Checking whether fortran needs dummy main")

    old = copy.deepcopy(conf.env["F77_LINKFLAGS"])
    try:
        conf.env["F77_LINKFLAGS"].extend(conf.env[FC_RUNTIME_LDFLAGS])
        ret = conf.builders["ctasks"].try_program("check_fc_dummy_main",
                code_tpl % {"main": "FC_DUMMY_MAIN"})
        if ret:
            conf.end_message("none")
            conf.env[FC_DUMMY_MAIN] = None
            return True
        else:
            conf.end_message("failed !")
            return False
    finally:
        conf.env["F77_LINKFLAGS"] = old

def check_fortran_mangling(conf):
    subr = """
      subroutine foobar()
      return
      end
      subroutine foo_bar()
      return
      end
"""
    main_tmpl = """
      int %s() { return 1; }
"""
    prog_tmpl = """
      void %(foobar)s(void);
      void %(foo_bar)s(void);
      int main() {
      %(foobar)s();
      %(foo_bar)s();
      return 0;
      }
"""

    conf.start_message("Checking fortran mangling scheme")
    old = {}
    for k in ["F77_LINKFLAGS", "LIBS", "LIBDIR"]:
        old[k] = copy.deepcopy(conf.env[k])
    try:
        mangling_lib = "check_fc_mangling_lib"
        ret = conf.builders["fortran"].try_static_library(mangling_lib, subr)
        if ret:
            if conf.env[FC_DUMMY_MAIN] is not None:
                main = main_tmpl % conf.env["FC_DUMMY_MAIN"]
            else:
                main = ""
            conf.env["LIBS"].insert(0, mangling_lib)
            libdir = conf.last_task.outputs[-1].parent.abspath()
            conf.env["LIBDIR"].insert(0, libdir)

            for u, du, case in mangling_generator():
                names = {"foobar": mangle_func("foobar", u, du, case),
                         "foo_bar": mangle_func("foo_bar", u, du, case)}
                prog = prog_tmpl % names
                name = "check_fc_mangling_main"
                def _name(u):
                    if u == "_":
                        return "u"
                    else:
                        return "nu"
                name += "_%s_%s_%s" % (_name(u), _name(du), case)
                ret = conf.builders["ctasks"].try_program(name, main + prog)
                if ret:
                    conf.env["FC_MANGLING"] = (u, du, case)
                    conf.end_message("%r %r %r" % (u, du, case))
                    return
            conf.end_message("failed !")
            conf.fail_configuration(None)
        else:
            conf.end_message("failed !")
            conf.fail_configuration(None)

    finally:
        for k in old:
            conf.env[k] = old[k]

def mangling_generator():
    for under in ['_', '']:
        for double_under in ['', '_']:
            for case in ["lower", "upper"]:
                yield under, double_under, case

def mangle_func(name, under, double_under, case):
    return getattr(name, case)() + under + (name.find("_") != -1 and double_under or '')



def monkey_patch():
    # XXX: keep the import here to avoid any side-effects from mere import of
    # bento.distutils
    from bento.distutils.utils \
        import \
            _is_setuptools_activated
    import bento.distutils.dist

    # Install it throughout the distutils
    _MODULES = []
    if _is_setuptools_activated():
        import setuptools.dist
        _MODULES.append(setuptools.dist)
    import distutils.dist, distutils.core, distutils.cmd
    _MODULES.extend([distutils.dist, distutils.core, distutils.cmd])
    for module in _MODULES:
        module.Distribution = bento.distutils.dist.BentoDistribution



class BentoError(Exception):
    pass

class InternalBentoError(BentoError):
    def __str__(self):
        return "unexpected error: %s (most likely a bento bug)"

class InvalidPackage(BentoError):
    pass

class UsageException(BentoError):
    pass

class ParseError(BentoError):
    def __init__(self, msg="", token=None):
        self.msg = msg
        self.token = token
        if token is not None:
            self.lineno = token.lineno
            self.tp = token.type
            self.value = token.value
        else:
            self.lineno = self.tp = self.value = None
        self.filename = None
        # Don't use super here because Exception are not new-class object on 2.4
        BentoError.__init__(self, msg)

    def __str__(self):
        if self.filename is None or self.token is None:
            return Exception.__str__(self)
        else:
            msg = '  File "%(file)s", line %(lineno)d\n' \
                  % {"file": self.filename, "lineno": self.lineno}
            f = open(self.filename)
            try:
                # This is expensive, there must be a better way
                cnt = f.read()
                lines = cnt.splitlines()
                linepos = sum([(len(l)+1) for l in lines[:self.lineno-1]])
                msg += "%s\n" % lines[self.lineno-1]
                pos = self.token.lexpos
                msg += " " * (pos - linepos) + "^\n"
                msg += "Syntax error"
                return msg
            finally:
                f.close()

    def __repr__(self):
        return self.__str__()

class InvalidHook(BentoError):
    pass

class CommandExecutionFailure(BentoError):
    pass

class ConfigurationError(CommandExecutionFailure):
    pass

class BuildError(CommandExecutionFailure):
    pass

class ConvertionError(BentoError):
    pass

class SetupCannotRun(ConvertionError):
    pass

class UnsupportedFeature(ConvertionError):
    pass

class InvalidPyPIConfig(BentoError):
    pass

class PyPIError(BentoError):
    pass

class InvalidRepository(PyPIError):
    pass



import os
import sys
import re

from bento._config \
    import \
        _CLI

import six

SYS_EXECUTABLE = os.path.normpath(sys.executable)

SCRIPT_TEXT = """\
# BENTO AUTOGENERATED-CONSOLE SCRIPT
if __name__ == '__main__':
    import sys
    from %(module)s import %(function)s
    sys.exit(%(function)s())
"""

_LAUNCHER_MANIFEST = """
<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
<assembly xmlns="urn:schemas-microsoft-com:asm.v1" manifestVersion="1.0">
 <assemblyIdentity version="1.0.0.0"
 processorArchitecture="X86"
 name="%s.exe"
 type="win32"/>

 <!-- Identify the application security requirements. -->
 <trustInfo xmlns="urn:schemas-microsoft-com:asm.v3">
 <security>
 <requestedPrivileges>
 <requestedExecutionLevel level="asInvoker" uiAccess="false"/>
 </requestedPrivileges>
 </security>
 </trustInfo>
</assembly>"""

# XXX: taken from setuptools, audit this
def nt_quote_arg(arg):
    """Quote a command line argument according to Windows parsing
    rules"""

    result = []
    needquote = False
    nb = 0

    needquote = (" " in arg) or ("\t" in arg)
    if needquote:
        result.append('"')

    for c in arg:
        if c == '\\':
            nb += 1
        elif c == '"':
            # double preceding backslashes, then add a \"
            result.append('\\' * (nb*2) + '\\"')
            nb = 0
        else:
            if nb:
                result.append('\\' * nb)
                nb = 0
            result.append(c)

    if nb:
        result.append('\\' * nb)

    if needquote:
        result.append('\\' * nb)    # double the trailing backslashes
        result.append('"')

    return ''.join(result)

# XXX: taken verbatim from setuptools, rewrite this crap
def get_script_header(executable=SYS_EXECUTABLE, wininst=False):
    from distutils.command.build_scripts import first_line_re

    match = first_line_re.match(six.b(""))

    options = ''
    if match:
        options = match.group(1) or ''
        if options:
            options = ' ' + options
    if wininst:
        executable = "python.exe"
    else:
        executable = nt_quote_arg(executable)

    hdr = "#!%(executable)s%(options)s\n" % locals()
    if six.u(hdr).encode('ascii') != hdr:
        # Non-ascii path to sys.executable, use -x to prevent warnings
        if options:
            if options.strip().startswith('-'):
                options = ' -x' + options.strip()[1:]
            # else: punt, we can't do it, let the warning happen anyway
        else:
            options = ' -x'
    #executable = fix_jython_executable(executable, options)
    hdr = "#!%(executable)s%(options)s\n" % locals()
    return hdr

def create_scripts(executables, bdir):
    ret = {}

    for name, executable in executables.items():
        if sys.platform == "win32":
            ret[name] = create_win32_script(name, executable, bdir)
        else:
            ret[name] = create_posix_script(name, executable, bdir)
    return ret

def create_win32_script(name, executable, scripts_node):
    script_text = SCRIPT_TEXT % {"python_exec": SYS_EXECUTABLE,
            "module": executable.module,
            "function": executable.function}

    wininst = False
    header = get_script_header(SYS_EXECUTABLE, wininst)

    ext = '-script.py'
    launcher = _CLI

    new_header = re.sub('(?i)pythonw.exe', 'python.exe', header)

    if os.path.exists(new_header[2:-1]) or sys.platform != 'win32':
        hdr = new_header
    else:
        hdr = header

    fid = open(launcher, "rb")
    try:
        cnt = fid.read()
    finally:
        fid.close()

    def _write(name, cnt, mode):
        target = scripts_node.make_node(name)
        target.safe_write(cnt, "w%s" % mode)
        return target

    nodes = []
    nodes.append(_write(name + ext, hdr + script_text, 't'))
    nodes.append(_write(name + ".exe", cnt, 'b'))
    nodes.append(_write(name + ".exe.manifest", _LAUNCHER_MANIFEST % (name,), 't'))

    return nodes

def create_posix_script(name, executable, scripts_node):
    header = "#!%(python_exec)s\n" % {"python_exec": SYS_EXECUTABLE}
    cnt = SCRIPT_TEXT % {"python_exec": SYS_EXECUTABLE,
            "module": executable.module,
            "function": executable.function}

    n = scripts_node.make_node(name)
    n.safe_write(header + cnt)
    return [n]



import os
import tempfile
import shutil
from unittest import TestCase

class TmpContextBase(TestCase):
    def setUp(self):
        self.cwd = os.getcwd()
        self.d = tempfile.mkdtemp()
        os.chdir(self.d)

    def tearDown(self):
        shutil.rmtree(self.d)
        os.chdir(self.cwd)



"""
Ripped off from waf (v 1.6), by Thomas Nagy

The cool design is his, bugs most certainly mine :)
"""

import os, shutil, re, sys

def split_path(path):
    return path.split('/')

def split_path_cygwin(path):
    if path.startswith('//'):
        ret = path.split('/')[2:]
        ret[0] = '/' + ret[0]
        return ret
    return path.split('/')

re_sp = re.compile('[/\\\\]')
def split_path_win32(path):
    if path.startswith('\\\\'):
        ret = re.split(re_sp, path)[2:]
        ret[0] = '\\' + ret[0]
        return ret
    return re.split(re_sp, path)

if sys.platform == 'cygwin':
    split_path = split_path_cygwin
elif sys.platform == 'win32':
    split_path = split_path_win32

class Node(object):
    """
    This class is divided into two parts, the basic methods meant for filesystem access, and
    the ones which are bound to a context (the build context). The imports do not reflect this.
    """

    __slots__ = ('name', 'sig', 'children', 'parent', 'cache_abspath', 'cache_isdir')
    def __init__(self, name, parent):
        self.name = name
        self.parent = parent

        if parent:
            if name in parent.children:
                raise Errors.WafError('node %s exists in the parent files %r already' % (name, parent))
            parent.children[name] = self

    def __setstate__(self, data):
        self.name = data[0]
        self.parent = data[1]
        if data[2] is not None:
            self.children = data[2]
        if data[3] is not None:
            self.sig = data[3]

    def __getstate__(self):
        return (self.name, self.parent, getattr(self, 'children', None), getattr(self, 'sig', None))

    def __str__(self):
        return self.name

    def __repr__(self):
        return self.abspath()

    def __hash__(self):
        return id(self) # TODO see if it is still the case
        #raise Errors.WafError('do not hash nodes (too expensive)')

    def __eq__(self, node):
        return id(self) == id(node)

    def __copy__(self):
        "nodes are not supposed to be copied"
        raise Errors.WafError('nodes are not supposed to be copied')

    def read(self, flags='r'):
        "get the contents, assuming the node is a file"
        fid = open(self.abspath(), flags)
        try:
            return fid.read()
        finally:
            fid.close()

    def write(self, data, flags='w'):
        "write some text to the physical file, assuming the node is a file"
        f = None
        try:
            f = open(self.abspath(), flags)
            f.write(data)
        finally:
            if f:
                f.close()

    def chmod(self, val):
        "change file/dir permissions"
        os.chmod(self.abspath(), val)

    def delete(self):
        "delete the file physically, do not destroy the nodes"
        try:
            shutil.rmtree(self.abspath())
        except:
            pass

        try:
            delattr(self, 'children')
        except:
            pass

    def suffix(self):
        "scons-like - hot zone so do not touch"
        k = max(0, self.name.rfind('.'))
        return self.name[k:]

    def height(self):
        "amount of parents"
        d = self
        val = -1
        while d:
            d = d.parent
            val += 1
        return val

    def listdir(self):
        "list the directory contents"
        return os.listdir(self.abspath())

    def mkdir(self):
        "write a directory for the node"
        if getattr(self, 'cache_isdir', None):
            return

        try:
            self.parent.mkdir()
        except:
            pass

        if self.name:
            try:
                os.mkdir(self.abspath())
            except OSError:
                pass

            if not os.path.isdir(self.abspath()):
                raise IOError('%s is not a directory' % self)

            try:
                self.children
            except:
                self.children = {}

        self.cache_isdir = True

    def find_node(self, lst):
        "read the file system, make the nodes as needed"

        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']

        cur = self
        for x in lst:
            if x == '..':
                cur = cur.parent
                continue

            try:
                if x in cur.children:
                    cur = cur.children[x]
                    continue
            except:
                cur.children = {}

            # optimistic: create the node first then look if it was correct to do so
            cur = self.__class__(x, cur)
            try:
                os.stat(cur.abspath())
            except:
                del cur.parent.children[x]
                return None

        ret = cur

        try:
            while not getattr(cur.parent, 'cache_isdir', None):
                cur = cur.parent
                cur.cache_isdir = True
        except AttributeError:
            pass

        return ret

    def make_node(self, lst):
        "make a branch of nodes"
        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']

        cur = self
        for x in lst:
            if x == '..':
                cur = cur.parent
                continue

            if getattr(cur, 'children', {}):
                if x in cur.children:
                    cur = cur.children[x]
                    continue
            else:
                cur.children = {}
            cur = self.__class__(x, cur)
        return cur

    def search(self, lst):
        "dumb search for existing nodes"
        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']

        cur = self
        try:
            for x in lst:
                if x == '..':
                    cur = cur.parent
                else:
                    cur = cur.children[x]
            return cur
        except:
            pass

    def path_from(self, node):
        """path of this node seen from the other
            self = foo/bar/xyz.txt
            node = foo/stuff/
            -> ../bar/xyz.txt
        """
        # common root in rev 7673

        c1 = self
        c2 = node

        c1h = c1.height()
        c2h = c2.height()

        lst = []
        up = 0

        while c1h > c2h:
            lst.append(c1.name)
            c1 = c1.parent
            c1h -= 1

        while c2h > c1h:
            up += 1
            c2 = c2.parent
            c2h -= 1

        while id(c1) != id(c2):
            lst.append(c1.name)
            up += 1

            c1 = c1.parent
            c2 = c2.parent

        for i in range(up):
            lst.append('..')
        lst.reverse()
        return os.sep.join(lst) or '.'

    def abspath(self):
        """
        absolute path
        cache into the build context, cache_node_abspath
        """
        try:
            return self.cache_abspath
        except:
            pass
        # think twice before touching this (performance + complexity + correctness)
        if not self.parent:
            val = os.sep == '/' and os.sep or ''
        elif not self.parent.name:
            # drive letter for win32
            val = (os.sep == '/' and os.sep or '') + self.name
        else:
            val = self.parent.abspath() + os.sep + self.name

        self.cache_abspath = val
        return val

    def is_child_of(self, node):
        "does this node belong to the subtree node"
        p = self
        diff = self.height() - node.height()
        while diff > 0:
            diff -= 1
            p = p.parent
        return id(p) == id(node)

    # --------------------------------------------------------------------------------
    # the following methods require the source/build folders (bld.srcnode/bld.bldnode)
    # using a subclass is a possibility, but is that really necessary?
    # --------------------------------------------------------------------------------

    def is_src(self):
        """note: !is_src does not imply is_bld()"""
        cur = self
        x = id(self.ctx.srcnode)
        y = id(self.ctx.bldnode)
        while cur.parent:
            if id(cur) == y:
                return False
            if id(cur) == x:
                return True
            cur = cur.parent
        return False

    def is_bld(self):
        """note: !is_bld does not imply is_src"""
        cur = self
        y = id(self.ctx.bldnode)
        while cur.parent:
            if id(cur) == y:
                return True
            cur = cur.parent
        return False

    def get_src(self):
        """for a build node, will return the equivalent src node (or self if not possible)"""
        cur = self
        x = id(self.ctx.srcnode)
        y = id(self.ctx.bldnode)
        lst = []
        while cur.parent:
            if id(cur) == y:
                lst.reverse()
                return self.ctx.srcnode.make_node(lst)
            if id(cur) == x:
                return self
            lst.append(cur.name)
            cur = cur.parent
        return self

    def get_bld(self):
        """for a src node, will return the equivalent bld node (or self if not possible)"""
        cur = self
        x = id(self.ctx.srcnode)
        y = id(self.ctx.bldnode)
        lst = []
        while cur.parent:
            if id(cur) == y:
                return self
            if id(cur) == x:
                lst.reverse()
                return self.ctx.bldnode.make_node(lst)
            lst.append(cur.name)
            cur = cur.parent
        return self

    def find_resource(self, lst):
        """
        try to find a declared build node or a source file
        """
        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']
        elif issubclass(type(lst), type(self)):
            return lst

        node = self.get_bld().search(lst)
        if node:
            return node

        self = self.get_src()
        node = self.search(lst)
        if node:
            return node

        node = self.find_node(lst)
        if node:
            return node

        return node

    def find_or_declare(self, lst):
        """
        if 'self' is in build directory, try to return an existing node
        if no node is found, go to the source directory
        try to find an existing node in the source directory
        if no node is found, create it in the build directory
        """
        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']

        node = self.get_bld().search(lst)
        if node:
            if not os.path.isfile(node.abspath()):
                node.sig = None
                try:
                    node.parent.mkdir()
                except:
                    pass
            return node
        self = self.get_src()
        node = self.find_node(lst)
        if node:
            if not os.path.isfile(node.abspath()):
                node.sig = None
                try:
                    node.parent.mkdir()
                except:
                    pass
            return node
        node = self.get_bld().make_node(lst)
        node.parent.mkdir()
        return node

    def declare(self, lst):
        """
        if 'self' is in build directory, try to return an existing node
        if no node is found, create it in the build directory
        """
        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']

        node = self.get_bld().search(lst)
        if node:
            if not os.path.isfile(node.abspath()):
                node.sig = None
                try:
                    node.parent.mkdir()
                except:
                    pass
            return node
        node = self.get_bld().make_node(lst)
        node.parent.mkdir()
        return node

    def find_dir(self, lst):
        """
        search a folder in the filesystem
        create the corresponding mappings source <-> build directories
        """
        if isinstance(lst, str):
            lst = [x for x in split_path(lst) if x and x != '.']

        node = self.find_node(lst)
        try:
            os.path.isdir(node.abspath())
        except OSError:
            return None
        return node

    # helpers for building things
    def change_ext(self, ext):
        "node of the same path, but with a different extension - hot zone so do not touch"
        name = self.name
        # XXX: is using name.find(".") as done in waf a bug ?
        k = name.rfind('.')
        if k >= 0:
            name = name[:k] + ext
        else:
            name = name + ext

        return self.parent.find_or_declare([name])

    def nice_path(self, env=None):
        "printed in the console, open files easily from the launch directory"
        return self.path_from(self.ctx.launch_node())

    def bldpath(self):
        "path seen from the build directory default/src/foo.cpp"
        return self.path_from(self.ctx.bldnode)

    def srcpath(self):
        "path seen from the source directory ../src/foo.cpp"
        return self.path_from(self.ctx.srcnode)

    def relpath(self):
        "if a build node, bldpath, else srcpath"
        cur = self
        x = id(self.ctx.bldnode)
        while cur.parent:
            if id(cur) == x:
                return self.ctxpath()
            cur = cur.parent
        return self.srcpath()

    def bld_dir(self):
        "build path without the file name"
        return self.parent.bldpath()

    def bld_base(self):
        "build path without the extension: src/dir/foo(.cpp)"
        s = os.path.splitext(self.name)[0]
        return self.bld_dir() + os.sep + s



import os

from cPickle \
    import \
        load, dump

from yaku.utils \
    import \
        rename

CACHE_FILE = ".cache.lock"

class BuildContext(object):
    def __init__(self):
        self.object_tasks = []
        self.cache = {}
        self.env = {}

    def load(self):
        if os.path.exists(CACHE_FILE):
            fid = open(CACHE_FILE, "rb")
            try:
                self.cache = load(fid)
            finally:
                fid.close()
        else:
            self.cache = {}

    def save(self):
        # Use rename to avoid corrupting the cache if interrupted
        tmp_fid = open(CACHE_FILE + ".tmp", "w")
        try:
            dump(self.cache, tmp_fid)
        finally:
            tmp_fid.close()
        rename(CACHE_FILE + ".tmp", CACHE_FILE)

def get_bld():
    bld = BuildContext()
    bld.load()

    bld.env.update({
            "VERBOSE": False,
            "BLDDIR": "build"})

    return bld




import os

import warnings

from bento.core.pkg_objects \
    import \
        Extension
from bento.core.node \
    import \
        split_path

def translate_name(name, ref_node, from_node):
    if from_node != ref_node:
        parent_pkg = ref_node.path_from(from_node).replace(os.sep, ".")
        return ".".join([parent_pkg, name])
    else:
        return name

class NodeDataFiles(object):
    def __init__(self, name, nodes, ref_node, target_dir):
        self.name = name
        self.nodes = nodes
        self.ref_node = ref_node
        self.target_dir = target_dir

class NodeExtension(object):
    def __init__(self, name, nodes, top_node, ref_node, sub_directory_node=None):
        self.name = name
        self.top_node = top_node
        self.ref_node = ref_node
        self.nodes = nodes

        if sub_directory_node is None:
            self.top_or_lib_node = top_node
        else:
            self.top_or_lib_node = sub_directory_node

        if not ref_node.is_child_of(self.top_or_lib_node):
            self.full_name = name
        else:
            self.full_name = translate_name(name, ref_node, self.top_or_lib_node)

    def extension_from(self, from_node=None):
        if len(self.nodes) < 1:
            return Extension(self.name, [])
        else:
            if from_node is None:
                from_node = self.nodes[0].srcnode
            if not from_node.is_src():
                raise ValueError("node %s is not a source directory !" % from_node.abspath())
            if not self.ref_node.is_child_of(from_node):
                raise ValueError("from_node should be a parent of %s, but is %s" % \
                                 (self.ref_node.abspath(), from_node.abspath()))
            else:
                def translate_full_name(full_name):
                    parent_pkg = from_node.path_from(self.top_node)
                    if parent_pkg == ".":
                        parent_components = []
                    else:
                        parent_components = split_path(parent_pkg)
                    full_name_components = self.full_name.split(".")
                    if not full_name_components[:len(parent_components)] == parent_components:
                        raise ValueError("Internal bug: unexpected parent/name components: %s %s" % \
                                         (parent_components, full_name_components))
                    else:
                        return ".".join(full_name_components[len(parent_components):])
                relative_name = translate_full_name(self.full_name)
                return Extension(relative_name, sources=[n.path_from(from_node) for n in self.nodes])

class NodePythonPackage(object):
    def __init__(self, name, nodes, top_node, ref_node, sub_directory_node=None):
        self.nodes = nodes
        self.top_node = top_node
        self.ref_node = ref_node

        if sub_directory_node is None:
            self.top_or_lib_node = top_node
        else:
            self.top_or_lib_node = sub_directory_node

        if not ref_node.is_child_of(self.top_or_lib_node):
            raise IOError()

        self.full_name = translate_name(name, ref_node, self.top_or_lib_node)

class NodeRepresentation(object):
    """Node-based representation of a Package content."""
    def __init__(self, run_node, top_node, sub_directory_node=None):
        self.top_node = top_node
        self.run_node = run_node
        self.sub_directory_node = sub_directory_node

        if sub_directory_node is None:
            self.top_or_sub_directory_node = top_node
        else:
            if not sub_directory_node.is_child_of(top_node):
                raise IOError("sub_directory_node %r is not a subdirectory of %s" % \
                              (sub_directory_node, top_node))
            self.top_or_sub_directory_node = sub_directory_node

        self._registry = {}
        for category in ("modules", "packages", "extensions",
                         "compiled_libraries", "datafiles"):
            self._registry[category] = {}

        self._extra_source_nodes = []
        self._aliased_source_nodes = {}

    def to_node_extension(self, extension, source_node, ref_node):
        nodes = []
        for s in extension.sources:
            _nodes = source_node.ant_glob(s)
            if len(_nodes) < 1:
                #name = translate_name(extension.name, ref_node, self.top_or_sub_directory_node)
                raise IOError("Sources glob entry %r for extension %r did not return any result" \
                              % (s, extension.name))
            else:
                nodes.extend(_nodes)
        if extension.include_dirs:
            raise NotImplementedError("include dirs translation not implemented yet")
        return NodeExtension(extension.name, nodes, self.top_node, ref_node, self.sub_directory_node)

    def _run_in_subpackage(self, pkg, func):
        for name, sub_pkg in pkg.subpackages.items():
            ref_node = self.top_node.find_node(sub_pkg.rdir)
            if ref_node is None:
                raise IOError("directory %s relative to %s not found !" % (sub_pkg.rdir,
                              self.top_node.abspath()))
            func(sub_pkg, ref_node)

    def _update_extensions(self, pkg):
        for name, extension in pkg.extensions.items():
            ref_node = self.top_node
            extension = self.to_node_extension(extension, self.top_node, ref_node)
            self._registry["extensions"][extension.full_name] = extension

        def _subpackage_extension(sub_package, ref_node):
            for name, extension in sub_package.extensions.items():
                extension = self.to_node_extension(extension, ref_node, ref_node)
                full_name = translate_name(name, ref_node, self.top_node)
                self._registry["extensions"][full_name] = extension
        self._run_in_subpackage(pkg, _subpackage_extension)

    def _update_libraries(self, pkg):
        for name, compiled_library in pkg.compiled_libraries.items():
            ref_node = self.top_node
            compiled_library = self.to_node_extension(compiled_library, self.top_node, ref_node)
            self._registry["compiled_libraries"][name] = compiled_library

        def _subpackage_compiled_libraries(sub_package, ref_node):
            for name, compiled_library in sub_package.compiled_libraries.items():
                compiled_library = self.to_node_extension(compiled_library, ref_node, ref_node)
                name = translate_name(name, ref_node, self.top_node)
                self._registry["compiled_libraries"][name] = compiled_library
        self._run_in_subpackage(pkg, _subpackage_compiled_libraries)

    def _update_py_packages(self, pkg):
        def _resolve_package(package_name, ref_node):
            init = os.path.join(*(package_name.split(".") + ["__init__.py"]))
            n = ref_node.find_node(init)
            if n is None:
                raise IOError("init file for package %s not found (looked for %r)!" \
                              % (package_name, init))
            else:
                p = n.parent
                nodes = [p.find_node(f) for f in p.listdir() if f.endswith(".py")]
                node_package = NodePythonPackage(package_name, nodes, self.top_node,
                                                 ref_node, self.sub_directory_node)
                self._registry["packages"][node_package.full_name] = node_package

        def _subpackage_resolve_package(sub_package, ref_node):
            for package in sub_package.packages:
                _resolve_package(package, ref_node)

        for package in pkg.packages:
            _resolve_package(package, self.top_or_sub_directory_node)
        self._run_in_subpackage(pkg, _subpackage_resolve_package)

    def _update_data_files(self, pkg):
        for name, data_section in pkg.data_files.items():
            ref_node = self.top_node.find_node(data_section.source_dir)
            nodes = []
            for f in data_section.files:
                ns = ref_node.ant_glob(f)
                if len(ns) < 1:
                    raise IOError("File/glob %s could not be resolved (data file section %s)" % (f, name))
                else:
                    nodes.extend(ns)
            self._registry["datafiles"][name] = NodeDataFiles(name, nodes, ref_node, data_section.target_dir)

    def _update_py_modules(self, pkg):
        for m in pkg.py_modules:
            n = self.top_or_sub_directory_node.find_node("%s.py" % m)
            if n is None:
                raise IOError("file for module %s not found" % m)
            else:
                self._registry["modules"][m] = n

    def _update_extra_sources(self, pkg):
        for s in pkg.extra_source_files:
            nodes = self.top_node.ant_glob(s)
            if len(nodes) < 1:
                warnings.warn("extra source files glob entry %r did not return any result" % (s,))
            self._extra_source_nodes.extend(nodes)

    def update_package(self, pkg):
        self._update_py_packages(pkg)
        self._update_py_modules(pkg)

        self._update_extensions(pkg)
        self._update_libraries(pkg)

        self._update_data_files(pkg)
        self._update_extra_sources(pkg)

    def iter_category(self, category):
        if category in self._registry:
            return self._registry[category].items()
        else:
            raise ValueError("Unknown category %s" % category)

    def register_entity(self, category, name, entity):
        if category in self._registry:
            self._registry[category][name] = entity
        else:
            raise ValueError("Category %r not registered" % category)

    def iter_source_nodes(self):
        for n in self._extra_source_nodes:
            yield n

        for d in self._registry["datafiles"].values():
            for n in d.nodes:
                yield n

        for m in self._registry["modules"].values():
            yield m
        for package in self._registry["packages"].values():
            for n in package.nodes:
                yield n

        for extension in self._registry["extensions"].values():
            for n in extension.nodes:
                yield n
        for compiled_library in self._registry["compiled_libraries"].values():
            for n in compiled_library.nodes:
                yield n

    def iter_source_files(self):
        for n in self.iter_source_nodes():
            filename = n.path_from(self.run_node)
            alias = self._aliased_source_nodes.get(n, filename)
            yield filename, alias



import os
import sys
import re
import glob

from os.path import \
    join, split, splitext, dirname

from yaku._config \
    import \
        _OUTPUT

# Color handling for terminals (taken from waf)
COLORS_LST = {
        'USE' : True,
        'BOLD'  :'\x1b[01;1m',
        'RED'   :'\x1b[01;31m',
        'GREEN' :'\x1b[32m',
        'YELLOW':'\x1b[33m',
        'PINK'  :'\x1b[35m',
        'BLUE'  :'\x1b[01;34m',
        'CYAN'  :'\x1b[36m',
        'NORMAL':'\x1b[0m',
        'cursor_on'  :'\x1b[?25h',
        'cursor_off' :'\x1b[?25l',
}

GOT_TTY = not os.environ.get('TERM', 'dumb') in ['dumb', 'emacs']
if GOT_TTY:
    try:
        GOT_TTY = _OUTPUT.isatty()
    except AttributeError:
        GOT_TTY = False
if not GOT_TTY or 'NOCOLOR' in os.environ:
    COLORS_LST['USE'] = False

def get_color(cl):
    if not COLORS_LST['USE']:
        return ''
    return COLORS_LST.get(cl, '')

class foo(object):
    def __getattr__(self, a):
        return get_color(a)
    def __call__(self, a):
        return get_color(a)
COLORS = foo()

def pprint(color, str):
    _OUTPUT.write('%s%s%s\n' % (COLORS(color), str, COLORS('NORMAL')))



import os

import os.path as op


def find_root(p):
    """Return the 'root' of the given path.

    Example
    -------
    >>> find_root("/Users/joe")
    '/'
    """
    while p != op.dirname(p):
        p = op.dirname(p)
    return p

def ensure_dir(path):
    d = op.dirname(path)
    if d and not op.exists(d):
        os.makedirs(d)

def unnormalize_path(path):
    if os.sep != "/":
        return path.replace("/", "\\")
    else:
        return path

def normalize_path(path):
    if os.sep != "/":
        return path.replace("\\", "/")
    else:
        return path



import os
import sys

from yaku.task_manager \
    import \
        extension, get_extension_hook
from yaku.task \
    import \
        task_factory
from yaku.compiled_fun \
    import \
        compile_fun
from yaku.utils \
    import \
        ensure_dir, find_program
import yaku.errors

@extension(".pyx")
def cython_hook(self, node):
    self.sources.append(node.change_ext(".c"))
    return cython_task(self, node)

def cython_task(self, node):
    out = node.change_ext(".c")
    target = node.parent.declare(out.name)
    ensure_dir(target.name)

    task = task_factory("cython")(inputs=[node], outputs=[target])
    task.gen = self
    task.env_vars = []
    task.env = self.env

    self.env["CYTHON_INCPATH"] = ["-I%s" % p for p in
                self.env["CYTHON_CPPPATH"]]
    task.func = compile_fun("cython", "${CYTHON} ${SRC} -o ${TGT} ${CYTHON_INCPATH}",
                            False)[0]
    return [task]

class CythonBuilder(yaku.tools.Builder):
    def __init__(self, ctx):
        yaku.tools.Builder.__init__(self, ctx)

    def configure(self):
        ctx = self.ctx
        sys.stderr.write("Looking for cython... ")
        if detect(ctx):
            sys.stderr.write("yes\n")
        else:
            sys.stderr.write("no!\n")
            raise yaku.errors.ToolNotFound()
        ctx.env["CYTHON_CPPPATH"] = []
        ctx.env["CYTHON"] = [sys.executable, "-m", "cython"]
        ctx.env["ENV"]["PYTHONPATH"] = os.environ["PYTHONPATH"]

def detect(ctx):
    if find_program("cython") is None:
        return False
    else:
        return True

def get_builder(ctx):
    return CythonBuilder(ctx)



"""
Cache version 1

db["version"] : version number
db["magic"]   : "BENTOMAGIC"
db["bento_checkums"] : pickled dictionary {filename: checksum(filename)} for
                       each bento.info (including subentos)
db["package_description"] : pickled PackageDescription instance
db["user_flags"] : pickled user_flags dict
db["parsed_dict"]: pickled raw parsed dictionary (as returned by
                   raw_parse, before having been seen by the visitor)
"""
import os
import sys
import warnings

from bento.parser.misc \
    import \
        raw_parse
from bento.core.package \
    import \
        raw_to_pkg_kw, PackageDescription
from bento.core.options \
    import \
        raw_to_options_kw, PackageOptions
from bento.utils.utils import extract_exception
import bento.utils.path
import bento.utils.io2

if sys.version_info[0] < 3:
    import cPickle as pickle
else:
    import pickle

try:
    from hashlib import md5
except ImportError:
    from md5 import md5


class CachedPackage(object):
    def __init__(self, db_node):
        self._db_location = db_node

    def get_package(self, bento_info, user_flags=None):
        cache = _CachedPackageImpl(self._db_location.abspath())
        try:
            return cache.get_package(bento_info, user_flags)
        finally:
            cache.close()

    def get_options(self, bento_info):
        cache = _CachedPackageImpl(self._db_location.abspath())
        try:
            return cache.get_options(bento_info)
        finally:
            cache.close()

class _CachedPackageImpl(object):
    __version__ = "2"
    __magic__ = "CACHED_PACKAGE_BENTOMAGIC"

    def _has_valid_magic(self, db):
        try:
            magic = db["magic"]
            if not magic == self.__magic__:
                return False
            else:
                return True
        except KeyError:
            return False

    def _reset(self):
        self.db = {}
        self.db["magic"] = self.__magic__
        self.db["version"] = self.__version__
        self._first_time = True

    def _load_existing_cache(self, db_location):
        fid = open(db_location, "rb")
        try:
            db = pickle.load(fid)
            if not self._has_valid_magic(db):
                warnings.warn("Resetting invalid cached db")
                self._reset()
        finally:
            fid.close()

        version = db["version"]
        if version != self.__version__:
            warnings.warn("Resetting invalid version of cached db")
            self._reset()

        return db

    def __init__(self, db_location):
        self._location = db_location
        self._first_time = False
        if not os.path.exists(db_location):
            bento.utils.path.ensure_dir(db_location)
            self._reset()
        else:
            try:
                self.db = self._load_existing_cache(db_location)
            except Exception:
                e = extract_exception()
                warnings.warn("Resetting invalid cached db: (reason: %r)" % e)
                self._reset()

    def _has_invalidated_cache(self):
        if "bentos_checksums" in self.db:
            r_checksums = pickle.loads(self.db["bentos_checksums"])
            for f in r_checksums:
                checksum = md5(open(f, "rb").read()).hexdigest()
                if checksum != r_checksums[f]:
                    return True
            return False
        else:
            return True

    def get_package(self, bento_info, user_flags=None):
        try:
            return self._get_package(bento_info, user_flags)
        except Exception:
            e = extract_exception()
            warnings.warn("Resetting invalid cache (error was %r)" % e)
            self._reset()
            return self._get_package(bento_info, user_flags)

    def _get_package(self, bento_info, user_flags=None):
        if self._first_time:
            self._first_time = False
            return _create_package_nocached(bento_info, user_flags, self.db)
        else:
            if self._has_invalidated_cache():
                return _create_package_nocached(bento_info, user_flags, self.db)
            else:
                r_user_flags = pickle.loads(self.db["user_flags"])
                if user_flags is None:
                    # FIXME: this case is wrong
                    return pickle.loads(self.db["package_description"])
                elif r_user_flags != user_flags:
                    return _create_package_nocached(bento_info, user_flags, self.db)
                else:
                    raw = pickle.loads(self.db["parsed_dict"])
                    pkg, files = _raw_to_pkg(raw, user_flags, bento_info)
                    return pkg

    def get_options(self, bento_info):
        try:
            return self._get_options(bento_info)
        except Exception:
            e = extract_exception()
            warnings.warn("Resetting invalid cache (error was %r)" % e)
            self._reset()
            return self._get_options(bento_info)

    def _get_options(self, bento_info):
        if self._first_time:
            self._first_time = False
            return _create_options_nocached(bento_info, {}, self.db)
        else:
            if self._has_invalidated_cache():
                return _create_options_nocached(bento_info, {}, self.db)
            else:
                raw = pickle.loads(self.db["parsed_dict"])
                return _raw_to_options(raw)

    def close(self):
        bento.utils.io2.safe_write(self._location, lambda fd: pickle.dump(self.db, fd))

def _create_package_nocached(bento_info, user_flags, db):
    pkg, options = _create_objects_no_cached(bento_info, user_flags, db)
    return pkg

def _create_options_nocached(bento_info, user_flags, db):
    pkg, options = _create_objects_no_cached(bento_info, user_flags, db)
    return options

def _raw_to_options(raw):
    kw = raw_to_options_kw(raw)
    return PackageOptions(**kw)

def _raw_to_pkg(raw, user_flags, bento_info):
    kw, files = raw_to_pkg_kw(raw, user_flags, bento_info)
    pkg = PackageDescription(**kw)
    return pkg, files

def _create_objects_no_cached(bento_info, user_flags, db):
    d = os.path.dirname(bento_info.abspath())
    info_file = open(bento_info.abspath(), 'r')
    try:
        data = info_file.read()
        raw = raw_parse(data, bento_info.abspath())

        pkg, files = _raw_to_pkg(raw, user_flags, bento_info)
        files = [os.path.join(d, f) for f in files]
        options = _raw_to_options(raw)

        checksums = [md5(open(f, "rb").read()).hexdigest() for f in files]
        db["bentos_checksums"] = pickle.dumps(dict(zip(files, checksums)))
        db["package_description"] = pickle.dumps(pkg)
        db["user_flags"] = pickle.dumps(user_flags)
        db["parsed_dict"] = pickle.dumps(raw)

        return pkg, options
    finally:
        info_file.close()



import os

from bento.core.pkg_objects \
    import \
        Extension, CompiledLibrary

class SubPackageDescription:
    def __init__(self, rdir, packages=None, extensions=None,
                 compiled_libraries=None, py_modules=None, hook_files=None):
        self.rdir = rdir
        if packages is None:
            self.packages = []
        else:
            self.packages = packages
        if extensions is None:
            self.extensions = {}
        else:
            self.extensions = extensions
        if compiled_libraries is None:
            self.compiled_libraries = {}
        else:
            self.compiled_libraries = compiled_libraries
        if py_modules is None:
            self.py_modules = []
        else:
            self.py_modules = py_modules

        if hook_files is None:
            self.hook_files = []
        else:
            self.hook_files = hook_files

    def __repr__(self):
        return repr({"packages": self.packages,
                     "py_modules": self.py_modules,
                     "clibs": self.compiled_libraries,
                     "extensions": self.extensions})

def flatten_subpackage_packages(spkg, top_node):
    """Translate the (python) packages from a subpackage relatively to
    the given top node.
    """
    local_node = top_node.find_dir(spkg.rdir)
    parent_pkg = local_node.path_from(top_node).replace(os.pathsep, ".")
    ret = ["%s.%s" % (parent_pkg, p) for p in spkg.packages]
    return ret

def flatten_subpackage_extensions(spkg, top_node):
    """Translate the extensions from a subpackage relatively to the
    given top node.

    Extension name, source files and include directories paths are all
    translated relatively to the top node.

    Returns
    -------
    d : dict
        {ext_name: ext} dictionary

    Example
    -------
    Defining in /foo/bar the extension::

        Extension("_hello", sources=["src/hellomodule.c"])

    and top_node corresponding to /foo, the
    extension would be translated as::

        Extension("bar._hello", sources=["bar/src/hellomodule.c"])
    """
    local_node = top_node.find_dir(spkg.rdir)
    if local_node is None:
        raise IOError("Path %s not found" % \
                      os.path.join(top_node.abspath(), spkg.rdir))
    elif local_node == top_node:
        raise ValueError("Subpackage in top directory ??")

    ret = {}
    for name, extension in spkg.extensions.items():
        parent_pkg = spkg.rdir.replace(os.sep, ".")
        full_name = parent_pkg + ".%s" % name
        sources = []
        for s in extension.sources:
            node = local_node.find_node(s)
            if node is None:
                raise IOError("File %s not found" % s)
            sources.append(node.path_from(top_node))
        include_dirs = [
                local_node.find_node(d).path_from(top_node) \
                for d in extension.include_dirs]
        ret[full_name] = Extension(full_name, sources, include_dirs)
    return ret

def flatten_subpackage_compiled_libraries(spkg, top_node):
    """Translate the compiled libraries from a subpackage relatively
    to the given top node.

    Source files and include directories paths are all
    translated relatively to the top node.

    Returns
    -------
    d : dict
        {name: clib} dictionary

    Example
    -------
    Defining in /foo/bar the compiled library::

        CompiledLibrary("fubar", sources=["src/fubar.c"])

    and top_node corresponding to /foo, the
    extension would be translated as::

        CompiledLibrary("fubar", sources=["bar/src/fubar.c"])
    """
    local_node = top_node.find_dir(spkg.rdir)
    if local_node is None:
        raise IOError("Path %s not found" % \
                      os.path.join(top_node.abspath(), spkg.rdir))
    elif local_node == top_node:
        raise ValueError("Subpackage in top directory ??")

    ret = {}
    for name, clib in spkg.compiled_libraries.items():
        nodes = []
        for source in clib.sources:
            _nodes = local_node.ant_glob(source)
            if len(_nodes) < 1:
                raise IOError("Pattern %r did not resolve to anything !" % source)
            else:
                nodes.extend(_nodes)
        sources = [node.path_from(top_node) for node in nodes]
        include_dirs = [
                local_node.find_node(d).path_from(top_node) \
                for d in clib.include_dirs]
        parent_pkg = spkg.rdir.replace(os.sep, ".")
        full_name = ".".join([parent_pkg, name])
        ret[full_name] = CompiledLibrary(full_name, sources, include_dirs)
    return ret

def get_extensions(pkg, top_node):
    """Return the dictionary {name: extension} of all every extension
    in pkg, including the one defined in subpackages (if any).

    Note
    ----
    Extensions defined in subpackages are translated relatively to
    top_dir
    """
    extensions = {}
    for name, ext in pkg.extensions.items():
        extensions[name] = ext
    for spkg in pkg.subpackages.values():
        extensions.update(
                flatten_subpackage_extensions(spkg, top_node))
    return extensions

def get_compiled_libraries(pkg, top_node):
    """Return the dictionary {name: extension} of every compiled library in
    pkg, including the one defined in subpackages (if any).

    Note
    ----
    Extensions defined in subpackages are translated relatively to
    top_dir
    """
    libraries = {}
    for name, ext in pkg.compiled_libraries.items():
        libraries[name] = ext
    for spkg in pkg.subpackages.values():
        local_libs = flatten_subpackage_compiled_libraries(spkg,
                                                           top_node)
        libraries.update(local_libs)
    return libraries

def get_packages(pkg, top_node):
    """Return the dictionary {name: package} of every (python) package
    in pkg, including the one defined in subpackages (if any).
    """
    packages = [p for p in pkg.packages]
    for spkg in pkg.subpackages.values():
        local_pkgs = flatten_subpackage_packages(spkg, top_node)
        packages.extend(local_pkgs)
    return packages



from subprocess \
    import \
        call

# Code taken from python 2.6.5
class CalledProcessError(Exception):
    """This exception is raised when a process run by check_call() returns
    a non-zero exit status.  The exit status will be stored in the
    returncode attribute."""
    def __init__(self, returncode, cmd):
        self.returncode = returncode
        self.cmd = cmd
    def __str__(self):
        return "Command '%s' returned non-zero exit status %d" % (self.cmd, self.returncode)

def check_call(*popenargs, **kwargs):
    """Run command with arguments.  Wait for command to complete.  If
    the exit code was zero then return, otherwise raise
    CalledProcessError.  The CalledProcessError object will have the
    return code in the returncode attribute.

    The arguments are the same as for the Popen constructor.  Example:

    check_call(["ls", "-l"])
    """
    retcode = call(*popenargs, **kwargs)
    cmd = kwargs.get("args")
    if cmd is None:
        cmd = popenargs[0]
    if retcode:
        raise CalledProcessError(retcode, cmd)
    return retcode





import os
import tempfile
import shutil

import os.path as op

from bento.core \
    import \
        PackageDescription
from bento.compat.api.moves \
    import \
        unittest
from bento.core.node \
    import \
        create_base_nodes
from bento.core.testing \
    import \
        create_fake_package_from_bento_infos

from bento.commands.utils \
    import \
        has_cython_code

class TestIsUingCython(unittest.TestCase):
    def setUp(self):
        self.old_dir = os.getcwd()

        self.d = tempfile.mkdtemp()
        self.top_node, self.build_node, self.run_node = create_base_nodes(self.d, op.join(self.d, "build"))

        os.chdir(self.d)

    def tearDown(self):
        os.chdir(self.old_dir)
        shutil.rmtree(self.d)

    def test_simple_cython(self):
        bento_info = """\
Library:
    Extension: foo
        Sources: foo1.c, foo2.pyx
"""
        bentos = {"bento.info": bento_info}
        create_fake_package_from_bento_infos(self.run_node, bentos)

        pkg = PackageDescription.from_string(bento_info)
        self.assertTrue(has_cython_code(pkg))

    def test_simple_no_cython(self):
        bento_info = """\
Library:
    Extension: foo
        Sources: foo1.c, foo2.c
"""
        bentos = {"bento.info": bento_info}
        create_fake_package_from_bento_infos(self.run_node, bentos)

        pkg = PackageDescription.from_string(bento_info)
        self.assertFalse(has_cython_code(pkg))

    def test_sub_package(self):
        bento_info = """\
Recurse: foo

Library:
    Extension: foo
        Sources: foo1.c
"""
        bento_sub1_info = """
Library:
    Extension: bar
        Sources: bar.pyx
"""
        bentos = {"bento.info": bento_info, op.join("foo", "bento.info"): bento_sub1_info}
        create_fake_package_from_bento_infos(self.run_node, bentos)

        pkg = PackageDescription.from_string(bento_info)
        self.assertTrue(has_cython_code(pkg))



import os.path as op

from bento.distutils.tests.common \
    import \
        DistutilsCommandTestBase
from bento.distutils.dist \
    import \
        BentoDistribution
from bento.distutils.commands.install \
    import \
        install
        
class TestInstallCommand(DistutilsCommandTestBase):
    def test_simple(self):
        fid = open(op.join(self.new_cwd, "bento.info"), "wt")
        try:
            fid.write("""\
Name: foo
""")
        finally:
            fid.close()

        dist = BentoDistribution()
        cmd = install(dist)

        install_dir = op.join(self.new_cwd, "install")
        cmd.prefix = install_dir
        cmd.dry_run = True
        cmd.record = op.join(self.new_cwd, "record.txt")
        cmd.ensure_finalized()

        cmd.run()



import copy

def check_compiler(conf, msg=None):
    code = """\
int main(void)
{
    return 0;
}
"""

    if msg is None:
        conf.start_message("Checking whether C compiler works")
    else:
        conf.start_message(msg)
    ret = conf.builders["ctasks"].try_program("check_cc", code, None)
    if ret:
        conf.end_message("yes")
    else:
        conf.end_message("no")
    return ret

def check_cpp_symbol(conf, symbol, headers=None):
    code = []
    if headers:
        for h in headers:
            code.append(r"#include <%s>" % h)

    code.append(r"""
int main()
{
#ifndef %s
    (void) %s;
#endif
    ;
    return 0;
}
""" % (symbol, symbol))
    src = "\n".join(code)

    conf.start_message("Checking for declaration %s" % symbol)
    ret = conf.builders["ctasks"].try_compile("check_cpp_symbol", src, headers)
    conf.conf_results.append({"type": "decl", "value": symbol,
                              "result": ret})
    if ret:
        conf.end_message("yes")
    else:
        conf.end_message("no")
    return ret

def check_type(conf, type_name, headers=None):
    code = r"""
int main() {
  if ((%(name)s *) 0)
    return 0;
  if (sizeof (%(name)s))
    return 0;
}
""" % {'name': type_name}

    conf.start_message("Checking for type %s" % type_name)
    ret = conf.builders["ctasks"].try_compile("check_type", code,
                        headers)
    conf.conf_results.append({"type": "type", "value": type_name,
                              "result": ret})
    if ret:
        conf.end_message("yes")
    else:
        conf.end_message("no")
    return ret

def check_type_size(conf, type_name, headers=None, expect=None):
    """\
    This check can be used to get the size of a given type, or to
    check whether the type is of expected size.

    Arguments
    ---------
    conf : object
        configure context instance
    type_name : str
        the type to check
    includes : sequence
        list of headers to include in the test code before testing the
        type
    expect : sequence
        if given, will test wether the type has the given number of
            bytes.  If not given, will automatically find the size.
    """

    conf.start_message("Checking for sizeof %s ..." % type_name)
    body = r"""
typedef %(type)s yaku_check_sizeof_type;
int main ()
{
    static int test_array [1 - 2 * !(((long) (sizeof (yaku_check_sizeof_type))) >= 0)];
    test_array [0] = 0

    ;
    return 0;
}
""" % {"type": type_name}

    ret = conf.builders["ctasks"].try_compile("check_type_size", body, headers)
    if not ret:
        conf.end_message("no (cannot compile type)")
        return False

    if expect is None:
        # this fails to *compile* if size > sizeof(type)
        body = r"""
typedef %(type)s npy_check_sizeof_type;
int main ()
{
    static int test_array [1 - 2 * !(((long) (sizeof (npy_check_sizeof_type))) <= %(size)s)];
    test_array [0] = 0

    ;
    return 0;
}
"""
        # The principle is simple: we first find low and high bounds
        # of size for the type, where low/high are looked up on a log
        # scale. Then, we do a binary search to find the exact size
        # between low and high
        low = 0
        mid = 0
        while True:
            code = body % {'type': type_name, 'size': mid}
            ret = conf.builders["ctasks"].try_compile("check_type_size",
                    code, headers)
            if ret:
                break
            #log.info("failure to test for bound %d" % mid)
            low = mid + 1
            mid = 2 * mid + 1

        high = mid
        # Binary search:
        while low != high:
            mid = (high - low) / 2 + low
            code = body % {'type': type_name, 'size': mid}
            if conf.builders["ctasks"].try_compile("check_type_size",
                    code, headers):
                high = mid
            else:
                low = mid + 1
        ret = low
        size = low
        conf.end_message("%d" % low)
    else:
        raise NotImplementedError("Expect arg not yet implemented")

    conf.conf_results.append({"type": "type_size", "value": type_name,
                              "result": ret})
    return ret

def define(conf, name, value=None, comment=None):
    """\
    Define a new preprocessing symbol in the current config header.

    If value is None. then #define name is written. If value is not
    none, then #define name value is written.

    Parameters
    ----------
    conf: object
        Instance of the current configure context
    name: str
        Name of the symbol to define
    value : (None)
        If given and not None, the symbol will have this value
    comment: str
        will be put as a C comment in the header, to explain the
        meaning of the value (appropriate C comments /* and */ will be
        put automatically).
    """
    lines = []
    if comment:
        comment_str = "\n/* %s */" % comment
        lines.append(comment_str)
    else:
        lines.append("\n")

    if value is not None:
        define_str = "#define %s %s" % (name, value)
    else:
        define_str = "#define %s" % name
    lines.append(define_str)
    lines.append('')
    content = "\n".join(lines)

    conf.conf_results.append({"type": "define", "value": content,
        "result": True})

def check_header(conf, header):
    code = r"""
#include <%s>
""" % header

    conf.start_message("Checking for header %s" % header)
    ret = conf.builders["ctasks"].try_compile("check_header", code, None)
    if ret:
        conf.end_message("yes")
    else:
        conf.end_message("no !")
    conf.conf_results.append({"type": "header", "value": header,
                              "result": ret})
    return ret

def check_func(conf, func, libs=None):
    if libs is None:
        libs = []
    # Handle MSVC intrinsics: force MS compiler to make a function
    # call. Useful to test for some functions when built with
    # optimization on, to avoid build error because the intrinsic and
    # our 'fake' test declaration do not match.
    code = r"""
char %(func)s (void);

#ifdef _MSC_VER
#pragma function(%(func)s)
#endif

int main (void)
{
    return %(func)s();
}
""" % {"func": func}

    if libs:
        msg = "Checking for function %s in %s" % (func, " ".join([conf.env["LIB_FMT"] % lib for lib in libs]))
    else:
        msg = "Checking for function %s" % func
    conf.start_message(msg)

    old_lib = copy.deepcopy(conf.env["LIBS"])
    try:
        for lib in libs[::-1]:
            conf.env["LIBS"].insert(0, lib)
        ret = conf.builders["ctasks"].try_program("check_func", code, None)
        if ret:
            conf.end_message("yes")
        else:
            conf.end_message("no !")
    finally:
        conf.env["LIBS"] = old_lib
    conf.conf_results.append({"type": "func", "value": func,
                              "result": ret})
    return ret

def check_lib(conf, libs, func):
    # XXX: refactor with check_func

    # Handle MSVC intrinsics: force MS compiler to make a function
    # call. Useful to test for some functions when built with
    # optimization on, to avoid build error because the intrinsic and
    # our 'fake' test declaration do not match.
    code = r"""
char %(func)s (void);

#ifdef _MSC_VER
#pragma function(%(func)s)
#endif

int main (void)
{
    return %(func)s();
}
    """ % {"func": func}

    conf.start_message("Checking for function %s in %s" % \
                       (func, " ".join([conf.env["LIB_FMT"] % lib for lib in libs])))

    old_lib = copy.deepcopy(conf.env["LIBS"])
    try:
        for i in range(len(libs)):
            conf.env["LIBS"].insert(i, libs[i])
        ret = conf.builders["ctasks"].try_program("check_lib", code, None)
        if ret:
            conf.end_message("yes")
        else:
            conf.end_message("no !")
    finally:
        conf.env["LIBS"] = old_lib
    for lib in libs:
        conf.conf_results.append({"type": "lib", "value": lib,
                                  "result": ret, "func": func})
    return ret

def check_funcs_at_once(conf, funcs, libs=None):
    if libs is None:
        libs = []

    header = []
    header = ['#ifdef __cplusplus']
    header.append('extern "C" {')
    header.append('#endif')
    for f in funcs:
        header.append("\tchar %s();" % f)
        # Handle MSVC intrinsics: force MS compiler to make a function
        # call. Useful to test for some functions when built with
        # optimization on, to avoid build error because the intrinsic
        # and our 'fake' test declaration do not match.
        header.append("#ifdef _MSC_VER")
        header.append("#pragma function(%s)" % f)
        header.append("#endif")
    header.append('#ifdef __cplusplus')
    header.append('};')
    header.append('#endif')
    header = "\n".join(header)

    tmp = []
    for f in funcs:
        tmp.append("\t%s();" % f)
    tmp = "\n".join(tmp)

    body = r"""
%(include)s
%(header)s

int main (void)
{
    %(tmp)s
        return 0;
}
""" % {"tmp": tmp, "include": "", "header": header}

    conf.start_message("Checking for functions %s" % ", ".join(funcs))
    old_lib = copy.deepcopy(conf.env["LIBS"])
    try:
        for lib in libs[::-1]:
            conf.env["LIBS"].insert(0, lib)
        ret = conf.builders["ctasks"].try_program("check_func", body, None)
        if ret:
            conf.end_message("yes")
        else:
            conf.end_message("no !")
    finally:
        conf.env["LIBS"] = old_lib

    for func in funcs:
        conf.conf_results.append({"type": "func", "value": func,
                                  "result": ret})
    return ret



import copy
import os
import subprocess
import sys

import os.path as op

import bento.errors

from bento.commands.core \
    import \
        Command
from bento.commands.options \
    import \
        Option
from bento.utils.utils \
    import \
        extract_exception

_OUTPUT_DEFAULT = "html"

# FIXME: where to allow to customize this ?
_DOC_ROOT = "doc"
# relative to _DOC_ROOT
_SOURCE_ROOT = "source"

def guess_source_dir():
    for guess in ('doc', 'docs'):
        if not op.isdir(guess):
            continue
        for root, dirnames, filenames in os.walk(guess):
            if 'conf.py' in filenames:
                return root
    return None

class SphinxCommand(Command):
    long_descr = """\
Purpose: build sphinx documentation
Usage:   bentomaker sphinx [OPTIONS]."""
    short_descr = "build the project sphinx documentation."
    common_options = Command.common_options + \
                        [Option("--output-format",
                                help="Doc output format (default: %r)" % _OUTPUT_DEFAULT,
                                default=_OUTPUT_DEFAULT),
                         Option("--source-dir",
                             help="Doc source directory (guessed if not specified)"),
                         Option("--config-dir",
                             help="Config directory (guessed if not specified)"),
                         ]

    def can_run(self):
        try:
            import sphinx
            return True
        except ImportError:
            return False

    def run(self, context):
        if not self.can_run():
            return bento.errors.CommandExecutionFailure("sphinx not available")
        import sphinx.application

        p = context.options_context.parser
        o, a = p.parse_args(context.command_argv)
        if o.output_format != "html":
            raise ValueError("Only html output supported for now")
        else:
            builder = "html"

        if o.source_dir is None:
            source_dir = guess_source_dir()
        else:
            source_dir = o.source_dir
        if source_dir is None:
            raise bento.errors.UsageException("""\
Doc source dir could not be found: please specify the directory where your
sphinx conf.py is located with the --source-dir option""")
        if not op.isabs(source_dir):
            source_dir = op.join(context.top_node.abspath(), source_dir)

        sphinx_build = context.build_node.make_node("sphinx")
        html_build = sphinx_build.make_node(o.output_format)
        doctrees_build = sphinx_build.make_node("doctrees")

        doc_html_build = html_build.abspath()
        doc_doctrees_build = doctrees_build.abspath()

        confoverrides = {}
        status_stream = sys.stdout
        fresh_env = False
        force_all = False

        app = sphinx.application.Sphinx(
                source_dir, source_dir,
                doc_html_build, doc_doctrees_build,
                builder, confoverrides, status_stream,
                freshenv=fresh_env)
        try:
            app.build(force_all=force_all)
        except Exception:
            err = extract_exception()
            raise bento.errors.CommandExecutionFailure("error while building doc: %r" % str(err))
